// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function ResponseCacheInitialize(client, adminPort, server, elements, maxElementSize, state){
  var i;

  state.elements = elements;
  state.maxElementSize = maxElementSize;

  state.cachedElements = [];
  state.cachedElements.length = elements;
  for(i = 0; i < elements; i = i + 1){
    state.cachedElements[i] = {};
    state.cachedElements[i].cached = false;
  }

  state.idles = 0;
  state.requests = 0;
  state.cached = 0;
  state.duplicateHash = 0;
  state.responseTooLargeToCache = 0;

  return true;
}
function ResponseCacheIteration(client, adminPort, server, state, idleRef){
  var request, response;
  var hash;
  var cachedElement;
  var cached, clientReq, adminPortReq;
  var data;
  var adminPortOutput;
  var adminPortOutputJSON;
  var adminPortOutputBytes;
  var adminPortRequestBytes;

  request = {};
  response = {};

  if(ServerCheck(client)){
    clientReq = true;

    state.requests = state.requests + 1;

    ServerReceive(client, request);

    data = ByteArrayToNumberArray(request.bytes);
    hash = ModulusHash(data, state.elements);
    cachedElement = state.cachedElements[hash];

    cached = false;
    if(cachedElement.cached){
      if(ByteArraysEqual(request.bytes, cachedElement.request)){
        cached = true;
        state.cached = state.cached + 1;
      }else{
        state.duplicateHash = state.duplicateHash + 1;
      }
    }

    if( !cached ){
      Call(server, request.bytes, response);
      if(ByteArrayLength(response.bytes) <= state.maxElementSize){
        if(cachedElement.cached){
          delete(cachedElement.request);
          delete(cachedElement.response);
        }else{
          cachedElement.cached = true;
        }
        cachedElement.request = CopyByteArray(request.bytes);
        cachedElement.response = CopyByteArray(response.bytes);
      }else{
        state.responseTooLargeToCache = state.responseTooLargeToCache + 1;
      }
    }else{
      response.bytes = CopyByteArray(cachedElement.response);
    }

    ServerSend(client, response.bytes);
  }else{
    clientReq = false;
  }

  if(ServerCheck(adminPort)){
    adminPortReq = true;

    adminPortRequestBytes = {};
    ServerReceive(adminPort, adminPortRequestBytes);

    adminPortOutput = CreateStructData();
    AddNumberToStruct(adminPortOutput.structure, "idles".split(''), state.idles);
    AddNumberToStruct(adminPortOutput.structure, "Client Reqs".split(''), state.requests);
    AddNumberToStruct(adminPortOutput.structure, "Cached Reqs".split(''), state.cached);
    AddNumberToStruct(adminPortOutput.structure, "Duplicate Hash Req".split(''), state.duplicateHash);
    AddNumberToStruct(adminPortOutput.structure, "Over Cache Max Req".split(''), state.responseTooLargeToCache);

    adminPortOutputJSON = WriteJSON(adminPortOutput);

    adminPortOutputBytes = UTF16ToUTF8(adminPortOutputJSON);

    ServerSend(adminPort, adminPortOutputBytes);

    FreeData(adminPortOutput);
    delete(adminPortOutputJSON);
    FreeByteArray(adminPortOutputBytes);
  }else{
    adminPortReq = false;
  }

  if( !clientReq  &&  !adminPortReq ){
    idleRef.numberValue = 1;
    state.idles = state.idles + 1;
  }else{
    idleRef.numberValue = 0;
  }

  delete(request);
  delete(response);
}
function ModulusHash(array, m){
  var hash, i;

  hash = 0;

  for(i = 0; i < array.length; i = i + 1){
    hash = (hash + array[i])%m;
  }

  return hash;
}
function test(){
  var failures;

  failures = CreateNumberReference(0);

  TestResponseCacheIteration1(failures);

  return failures.numberValue;
}
function TestResponseCacheIteration1(failures){
  var client, adminPort;
  var server;
  var str;
  var state;
  var idleRef;

  state = {};
  idleRef = CreateNumberReference(0);

  adminPort = {};
  adminPort.requests = [];
  adminPort.requests.length = 1;
  adminPort.requests[0] = CreateNumberArrayReferenceLengthValue(0, 0);
  adminPort.requestCount = 0;
  adminPort.responseCount = 0;
  adminPort.responses = [];
  adminPort.responses.length = 1;
  adminPort.responses[0] = {};
  adminPort.check = [];
  adminPort.check.length = 2;
  adminPort.check[0] = 1;
  adminPort.check[1] = 20;

  client = {};
  client.requests = [];
  client.requests.length = 2;
  client.requests[0] = {};
  client.requests[0].numberArray = StringToNumberArray("request 1".split(''));
  client.requests[1] = {};
  client.requests[1].numberArray = StringToNumberArray("request 1".split(''));
  client.requestCount = 0;
  client.responseCount = 0;
  client.responses = [];
  client.responses.length = 2;
  client.check = [];
  client.check.length = 2;
  client.check[0] = 0;
  client.check[1] = 0;

  server = {};
  server.requests = [];
  server.requests.length = 2;
  server.requests[0] = {};
  server.requests[1] = {};
  server.requestCount = 0;
  server.responseCount = 0;
  server.responses = [];
  server.responses.length = 2;
  server.responses[0] = {};
  server.responses[0].numberArray = StringToNumberArray("response 1".split(''));
  server.responses[1] = {};
  server.responses[1].numberArray = StringToNumberArray("response 2".split(''));
  server.check = [];
  server.check.length = 1;
  server.check[0] = 0;

  ResponseCacheInitialize(client, adminPort, server, 10, 200, state);

  ResponseCacheIteration(client, adminPort, server, state, idleRef);
  ResponseCacheIteration(client, adminPort, server, state, idleRef);

  AssertEquals(client.responseCount, 2, failures);

  AssertEquals(server.requestCount, 1, failures);

  str = NumberArrayToString(server.requests[0].numberArray);
  AssertStringEquals(str, "request 1".split(''), failures);

  str = NumberArrayToString(client.responses[0].numberArray);
  AssertStringEquals(str, "response 1".split(''), failures);

  str = NumberArrayToString(client.responses[1].numberArray);
  AssertStringEquals(str, "response 1".split(''), failures);

  str = NumberArrayToString(adminPort.responses[0].numberArray);
  AssertStringEquals(str, "{\"idles\":0,\"Client Reqs\":2,\"Cached Reqs\":1,\"Duplicate Hash Req\":0,\"Over Cache Max Req\":0}".split(''), failures);
}
function Check(pu){
  var avaialble;
  var count;

  if(pu.responseCount < pu.check.length){
    count = pu.check[pu.responseCount];

    if(count <= 0){
      avaialble = true;
    }else{
      avaialble = false;
      count = count - 1;
    }

    pu.check[pu.responseCount] = count;
  }else{
    avaialble = true;
  }

  return avaialble;
}
function Send(pu, data){
  pu.requests[pu.requestCount] = {};
  pu.requests[pu.requestCount].numberArray = data.bytes;
  pu.requestCount = pu.requestCount + 1;
}
function Receive(pu, data){
  data.bytes = {};
  data.bytes.bytes = pu.responses[pu.responseCount].numberArray;
  pu.responseCount = pu.responseCount + 1;
}
function Call(pu, request, response){
  Send(pu, request);
  Receive(pu, response);
}
function ServerCheck(pu){
  var avaialble;
  var count;

  if(pu.requestCount < pu.check.length){
    count = pu.check[pu.requestCount];

    if(count <= 0){
      avaialble = true;
    }else{
      avaialble = false;
      count = count - 1;
    }

    pu.check[pu.requestCount] = count;
  }else{
    avaialble = true;
  }

  return avaialble;
}
function ServerSend(pu, data){
  pu.responses[pu.responseCount] = {};
  pu.responses[pu.responseCount].numberArray = data.bytes;
  pu.responseCount = pu.responseCount + 1;
}
function ServerReceive(pu, data){
  data.bytes = {};
  data.bytes.bytes = pu.requests[pu.requestCount].numberArray;
  pu.requestCount = pu.requestCount + 1;
}
function ReadXbytes(data, position, lengthx){
  var r;
  var i;

  r = CreateByteArray(lengthx);

  for(i = 0; i < lengthx; i = i + 1){
    r.bytes[i] = ReadByte(data, position);
  }

  return r;
}
function Read4bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 24)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function Read2bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function ReadByte(data, position){
  var next;

  next = data.bytes[position.numberValue];
  position.numberValue = position.numberValue + 1;

  return next;
}
function Read4bytesLE(data, position){
  var r;

  r = 0;
  r = r + ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 24)*ReadByte(data, position);

  return r;
}
function WriteByte(data, b, position){
  data.bytes[position.numberValue] = b;
  position.numberValue = position.numberValue + 1;
}
function Write2BytesLE(data, b, position){
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write4BytesLE(data, b, position){
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write2BytesBE(data, b, position){
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function Write4BytesBE(data, b, position){
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data.bytes[position.numberValue] = BytesRound(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function WriteStringBytes(data, cs, position){
  var i, v;

  for(i = 0; i < cs.length; i = i + 1){
    v = cs[i].charCodeAt(0);
    WriteByte(data, v, position);
  }
}
function BytesRound(x){
  return Math.floor(x + 0.5);
}
function ByteArrayToNumberArray(src){
  var dst;
  var i, len;

  len = src.bytes.length;
  dst = [];
  dst.length = len;
  for(i = 0; i < len; i = i + 1){
    dst[i] = src.bytes[i];
  }

  return dst;
}
function NumberArrayToByteArray(src){
  var dst;
  var i, len;

  len = src.length;
  dst = {};
  dst.bytes = [];
  dst.bytes.length = len;
  for(i = 0; i < len; i = i + 1){
    dst.bytes[i] = src[i];
  }

  return dst;
}
function ByteArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.bytes.length == b.bytes.length){
    for(i = 0; i < a.bytes.length && equal; i = i + 1){
      if(a.bytes[i] != b.bytes[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function CopyByteArray(a){
  var i;
  var n;

  n = {};
  n.bytes = [];
  n.bytes.length = a.bytes.length;

  for(i = 0; i < a.bytes.length; i = i + 1){
    n.bytes[i] = a.bytes[i];
  }

  return n;
}
function ByteArrayLength(response){
  return response.bytes.length;
}
function CreateAndFillByteArray(lengthx, value){
  var bytes;
  var i;

  bytes = {};
  bytes.bytes = [];
  bytes.bytes.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    bytes.bytes[i] = value;
  }

  return bytes;
}
function CreateByteArray(lengthx){
  var bytes;

  bytes = {};
  bytes.bytes = [];
  bytes.bytes.length = lengthx;

  return bytes;
}
function SetByte(array, index, value){
  array.bytes[index] = value;
}
function GetByte(array, index){
  return array.bytes[index];
}
function AssertByteArraysEqual(a, b, failures){
  var i;

  if(a.bytes.length == b.bytes.length){
    for(i = 0; i < a.bytes.length; i = i + 1){
      if(GetByte(a, i) != GetByte(b, i)){
        failures.numberValue = failures.numberValue + 1;
      }
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function FreeByteArray(byteArray){
  delete(byteArray.bytes);
  delete(byteArray);
}
function CopyByteArrayRange(a, from, to, b){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.bytes.length && to >= 0 && to <= a.bytes.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a.bytes[from + i];
    }

    b.bytes = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function BytesToTextBase16(bytes){
  var comb;
  var i, v;
  var r;

  r = [];
  r.length = bytes.bytes.length*2;
  comb = GenerateBase16ByteCombinations();

  for(i = 0; i < bytes.bytes.length; i = i + 1){
    v = bytes.bytes[i];
    r[2*i + 0] = comb[v].string[0];
    r[2*i + 1] = comb[v].string[1];
  }

  FreeBase64ByteCombinations(comb);

  return r;
}
function TextToBytesBase16WithCheck(string, bytes, message){
  var comb;
  var i, n, c, d;
  var reverseBase16;
  var codes;
  var success;

  codes = "0123456789ABCDEF".split('');

  reverseBase16 = [];
  reverseBase16.length = 128;
  for(i = 0; i < codes.length; i = i + 1){
    n = codes[i].charCodeAt(0);
    reverseBase16[n] = i;
  }

  if(string.length%2 == 0){
    bytes.bytes = CreateByteArray(string.length/2);
    comb = GenerateBase16ByteCombinations();

    success = true;
    for(i = 0; i < string.length/2 && success; i = i + 1){
      c = string[2*i + 0].charCodeAt(0);
      d = string[2*i + 1].charCodeAt(0);

      if(c == 'a'.charCodeAt(0)){
        c = 'A'.charCodeAt(0);
      }
      if(c == 'b'.charCodeAt(0)){
        c = 'B'.charCodeAt(0);
      }
      if(c == 'c'.charCodeAt(0)){
        c = 'C'.charCodeAt(0);
      }
      if(c == 'd'.charCodeAt(0)){
        c = 'D'.charCodeAt(0);
      }
      if(c == 'e'.charCodeAt(0)){
        c = 'E'.charCodeAt(0);
      }
      if(c == 'f'.charCodeAt(0)){
        c = 'F'.charCodeAt(0);
      }

      if(d == 'a'.charCodeAt(0)){
        d = 'A'.charCodeAt(0);
      }
      if(d == 'b'.charCodeAt(0)){
        d = 'B'.charCodeAt(0);
      }
      if(d == 'c'.charCodeAt(0)){
        d = 'C'.charCodeAt(0);
      }
      if(d == 'd'.charCodeAt(0)){
        d = 'D'.charCodeAt(0);
      }
      if(d == 'e'.charCodeAt(0)){
        d = 'E'.charCodeAt(0);
      }
      if(d == 'f'.charCodeAt(0)){
        d = 'F'.charCodeAt(0);
      }

      success = true;
      success = success && (c == '0'.charCodeAt(0) || c == '1'.charCodeAt(0) || c == '2'.charCodeAt(0) || c == '3'.charCodeAt(0) || c == '4'.charCodeAt(0) || c == '5'.charCodeAt(0) || c == '6'.charCodeAt(0) || c == '7'.charCodeAt(0) || c == '8'.charCodeAt(0) || c == '9'.charCodeAt(0) || c == 'A'.charCodeAt(0) || c == 'B'.charCodeAt(0) || c == 'C'.charCodeAt(0) || c == 'D'.charCodeAt(0) || c == 'E'.charCodeAt(0) || c == 'F'.charCodeAt(0));
      success = success && (d == '0'.charCodeAt(0) || d == '1'.charCodeAt(0) || d == '2'.charCodeAt(0) || d == '3'.charCodeAt(0) || d == '4'.charCodeAt(0) || d == '5'.charCodeAt(0) || d == '6'.charCodeAt(0) || d == '7'.charCodeAt(0) || d == '8'.charCodeAt(0) || d == '9'.charCodeAt(0) || d == 'A'.charCodeAt(0) || d == 'B'.charCodeAt(0) || d == 'C'.charCodeAt(0) || d == 'D'.charCodeAt(0) || d == 'E'.charCodeAt(0) || d == 'F'.charCodeAt(0));

      if(success){
        n = reverseBase16[c]*16 + reverseBase16[d];

        bytes.bytes.bytes[i] = n;
      }else{
        success = false;
        message.string = "Hex-string can only contain 0-9, A-F or a-f.".split('');
      }
    }

    FreeBase64ByteCombinations(comb);
  }else{
    bytes.bytes = CreateByteArray(0);
    success = false;
    message.string = "Hex-string must be an even number of characters.".split('');
  }

  delete(reverseBase16);

  return success;
}
function TextToBytesBase16(string){
  var comb;
  var i, n, c, d;
  var r;
  var reverseBase16;
  var codes;

  codes = "0123456789ABCDEF".split('');

  reverseBase16 = [];
  reverseBase16.length = 128;
  for(i = 0; i < codes.length; i = i + 1){
    n = codes[i].charCodeAt(0);
    reverseBase16[n] = i;
  }

  if(string.length%2 == 0){
    r = CreateByteArray(string.length/2);
    comb = GenerateBase16ByteCombinations();

    for(i = 0; i < string.length/2; i = i + 1){
      c = string[2*i + 0].charCodeAt(0);
      d = string[2*i + 1].charCodeAt(0);

      n = reverseBase16[c]*16 + reverseBase16[d];

      r.bytes[i] = n;
    }

    FreeBase64ByteCombinations(comb);
  }else{
    r = CreateByteArray(0);
  }

  delete(reverseBase16);

  return r;
}
function FreeBase64ByteCombinations(comb){
  var i;
  for(i = 0; i < 256; i = i + 1){
    FreeStringReference(comb[i]);
  }
  delete(comb);
}
function GenerateBase16ByteCombinations(){
  var comb;
  var codes;
  var i, j;
  comb = [];
  comb.length = 256;

  codes = "0123456789ABCDEF".split('');

  for(i = 0; i < 256; i = i + 1){
    comb[i] = {};
    comb[i].string = [];
    comb[i].string.length = 2;
  }

  for(j = 0; j < 16; j = j + 1){
    for(i = 0; i < 16; i = i + 1){
      comb[j*16 + i].string[1] = codes[i];
      comb[j*16 + i].string[0] = codes[j];
    }
  }

  return comb;
}
function JSONCompare(a, b, epsilon, equal, message){
  var success;
  var eaRef, ebRef;
  var ea, eb;

  eaRef = {};
  ebRef = {};

  success = ReadJSON(a, eaRef, message);

  if(success){
    ea = eaRef.data;

    success = ReadJSON(b, ebRef, message);

    if(success){
      eb = ebRef.data;

      equal.booleanValue = JSONCompareElements(ea, eb, epsilon);

      FreeData(eb);
    }

    FreeData(ea);
  }

  return success;
}
function JSONCompareElements(ea, eb, epsilon){
  var equal;

  equal = DataTypeEquals(ea, eb);
        
  if(equal){
    if(IsStructure(ea)){
      equal = JSONCompareObjects(ea.structure, eb.structure, epsilon);
    }else if(IsString(ea)){
      equal = StringsEqual(ea.string, eb.string);
    }else if(IsArray(ea)){
      equal = JSONCompareArrays(ea.array, eb.array, epsilon);
    }else if(IsNumber(ea)){
      equal = EpsilonCompare(ea.number, eb.number, epsilon);
    }else if(IsNoType(ea)){
      equal = true;
    }else if(IsBoolean(ea)){
      equal = ea.booleanx == eb.booleanx;
    }
  }
        
  return equal;
}
function JSONCompareArrays(ea, eb, epsilon){
  var equals;
  var i, lengthx;

  equals = ea.lengthx == eb.lengthx;

  if(equals){
    lengthx = ea.lengthx;
    for(i = 0; i < lengthx && equals; i = i + 1){
      equals = JSONCompareElements(ArrayIndex(ea, i), ArrayIndex(eb, i), epsilon);
    }
  }

  return equals;
}
function JSONCompareObjects(ea, eb, epsilon){
  var equals;
  var akeys, bkeys, i;
  var keys;
  var key;
  var aFoundReference, bFoundReference;
  var eaValue, ebValue;

  aFoundReference = {};
  bFoundReference = {};

  akeys = StructKeys(ea);
  bkeys = StructKeys(eb);

  equals = akeys == bkeys;

  if(equals){
    keys = GetStructKeys(ea);

    for(i = 0; i < keys.length && equals; i = i + 1){
      key = keys[i].string;

      eaValue = GetDataFromStructWithCheck(ea, key, aFoundReference);
      ebValue = GetDataFromStructWithCheck(eb, key, bFoundReference);

      if(aFoundReference.booleanValue && bFoundReference.booleanValue){
        equals = JSONCompareElements(eaValue, ebValue, epsilon);
      }else{
        equals = false;
      }
    }

    FreeStringReferenceArray(keys);
  }

  return equals;
}
function FreeStringReferenceArray(strings){
  var i;

  for(i = 0; i < strings.length; i = i + 1){
    delete(strings[i]);
  }
  delete(strings);
}
function WriteJSON(data){
  var result;
  var lengthx;
  var index;

  lengthx = ComputeJSONStringLength(data);
  result = [];
  result.length = lengthx;
  index = CreateNumberReference(0);

  if(IsStructure(data)){
    WriteObject(data, result, index);
  }else if(IsString(data)){
    WriteString(data, result, index);
  }else if(IsArray(data)){
    WriteArray(data, result, index);
  }else if(IsNumber(data)){
    WriteNumber(data, result, index);
  }else if(IsNoType(data)){
    strWriteStringToStingStream(result, index, "null".split(''));
  }else if(IsBoolean(data)){
    WriteBooleanValue(data, result, index);
  }

  return result;
}
function WriteBooleanValue(element, result, index){
  if(element.booleanx){
    strWriteStringToStingStream(result, index, "true".split(''));
  }else{
    strWriteStringToStingStream(result, index, "false".split(''));
  }
}
function WriteNumber(element, result, index){
  var numberString;

  if(element.number != 0){
    if(Math.abs(element.number) >= Math.pow(10, 15) || Math.abs(element.number) <= Math.pow(10,  -15)){
      numberString = nCreateStringScientificNotationDecimalFromNumber(element.number);
    }else{
      numberString = nCreateStringDecimalFromNumber(element.number);
    }
  }else{
    numberString = nCreateStringDecimalFromNumber(element.number);
  }

  strWriteStringToStingStream(result, index, numberString);
}
function WriteArray(data, result, index){
  var s;
  var entry;
  var i;

  strWriteStringToStingStream(result, index, "[".split(''));

  for(i = 0; i < data.array.lengthx; i = i + 1){
    entry = ArrayIndex(data.array, i);

    s = WriteJSON(entry);
    strWriteStringToStingStream(result, index, s);

    if(i + 1 != data.array.lengthx){
      strWriteStringToStingStream(result, index, ",".split(''));
    }
  }

  strWriteStringToStingStream(result, index, "]".split(''));
}
function WriteString(element, result, index){
  strWriteStringToStingStream(result, index, "\"".split(''));
  element.string = JSONEscapeString(element.string);
  strWriteStringToStingStream(result, index, element.string);
  strWriteStringToStingStream(result, index, "\"".split(''));
}
function JSONEscapeString(string){
  var i, lengthx;
  var index, lettersReference;
  var ns, escaped;

  lengthx = JSONEscapedStringLength(string);

  ns = [];
  ns.length = lengthx;
  index = CreateNumberReference(0);
  lettersReference = CreateNumberReference(0);

  for(i = 0; i < string.length; i = i + 1){
    if(JSONMustBeEscaped(string[i], lettersReference)){
      escaped = JSONEscapeCharacter(string[i]);
      strWriteStringToStingStream(ns, index, escaped);
    }else{
      strWriteCharacterToStingStream(ns, index, string[i]);
    }
  }

  return ns;
}
function JSONEscapedStringLength(string){
  var lettersReference;
  var i, lengthx;

  lettersReference = CreateNumberReference(0);
  lengthx = 0;

  for(i = 0; i < string.length; i = i + 1){
    if(JSONMustBeEscaped(string[i], lettersReference)){
      lengthx = lengthx + lettersReference.numberValue;
    }else{
      lengthx = lengthx + 1;
    }
  }
  return lengthx;
}
function JSONEscapeCharacter(c){
  var code;
  var escaped;
  var hexNumber;
  var q, rs, s, b, f, n, r, t;

  code = c.charCodeAt(0);

  q = 34;
  rs = 92;
  s = 47;
  b = 8;
  f = 12;
  n = 10;
  r = 13;
  t = 9;

  hexNumber = {};

  if(code == q){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = '\"';
  }else if(code == rs){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = '\\';
  }else if(code == s){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = '/';
  }else if(code == b){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = 'b';
  }else if(code == f){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = 'f';
  }else if(code == n){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = 'n';
  }else if(code == r){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = 'r';
  }else if(code == t){
    escaped = [];
    escaped.length = 2;
    escaped[0] = '\\';
    escaped[1] = 't';
  }else if(code >= 0 && code <= 31){
    escaped = [];
    escaped.length = 6;
    escaped[0] = '\\';
    escaped[1] = 'u';
    escaped[2] = '0';
    escaped[3] = '0';

    nCreateStringFromNumberWithCheck(code, 16, hexNumber);

    if(hexNumber.string.length == 1){
      escaped[4] = '0';
      escaped[5] = hexNumber.string[0];
    }else if(hexNumber.string.length == 2){
      escaped[4] = hexNumber.string[0];
      escaped[5] = hexNumber.string[1];
    }
  }else{
    escaped = [];
    escaped.length = 1;
    escaped[0] = c;
  }

  return escaped;
}
function JSONMustBeEscaped(c, letters){
  var code;
  var mustBeEscaped;
  var q, rs, s, b, f, n, r, t;

  code = c.charCodeAt(0);
  mustBeEscaped = false;

  q = 34;
  rs = 92;
  s = 47;
  b = 8;
  f = 12;
  n = 10;
  r = 13;
  t = 9;

  if(code == q || code == rs || code == s || code == b || code == f || code == n || code == r || code == t){
    mustBeEscaped = true;
    letters.numberValue = 2;
  }else if(code >= 0 && code <= 31){
    mustBeEscaped = true;
    letters.numberValue = 6;
  }else if(code >= Math.pow(2, 16)){
    mustBeEscaped = true;
    letters.numberValue = 6;
  }

  return mustBeEscaped;
}
function WriteObject(data, result, index){
  var s, key, escapedKey;
  var i;
  var keys;
  var objectElement;

  strWriteStringToStingStream(result, index, "{".split(''));

  keys = GetStructKeys(data.structure);
  for(i = 0; i < keys.length; i = i + 1){
    key = keys[i].string;
    objectElement = GetDataFromStruct(data.structure, key);

    escapedKey = JSONEscapeString(key);
    strWriteStringToStingStream(result, index, "\"".split(''));
    strWriteStringToStingStream(result, index, escapedKey);
    strWriteStringToStingStream(result, index, "\"".split(''));
    strWriteStringToStingStream(result, index, ":".split(''));

    s = WriteJSON(objectElement);
    strWriteStringToStingStream(result, index, s);

    if(i + 1 != keys.length){
      strWriteStringToStingStream(result, index, ",".split(''));
    }
  }

  strWriteStringToStingStream(result, index, "}".split(''));
}
function WriteJSONFromStruct(struct){
  var data;
  var json;

  data = CreateStructData();
  data.structure = struct;
  json = WriteJSON(data);

  return json;
}
function WriteJSONFromArray(array){
  var data;
  var json;

  data = CreateArrayData();
  data.array = array;
  json = WriteJSON(data);

  return json;
}
function ReadJSON(string, dataReference, message){
  var tokenArrayReference;
  var success;

  /* Tokenize. */
  tokenArrayReference = {};
  success = JSONTokenize(string, tokenArrayReference, message);

  if(success){
    /* Parse. */
    success = GetJSONValue(tokenArrayReference.stringArray, dataReference, message);
  }

  return success;
}
function GetJSONValue(tokens, dataReference, message){
  var i;
  var success;

  i = CreateNumberReference(0);
  success = GetJSONValueRecursive(tokens, i, 0, dataReference, message);

  return success;
}
function GetJSONValueRecursive(tokens, i, depth, dataReference, message){
  var str, substr, token;
  var stringToDecimalResult;
  var success;

  success = true;
  token = tokens[i.numberValue].string;

  if(StringsEqual(token, "{".split(''))){
    success = GetJSONObject(tokens, i, depth + 1, dataReference, message);
  }else if(StringsEqual(token, "[".split(''))){
    success = GetJSONArray(tokens, i, depth + 1, dataReference, message);
  }else if(StringsEqual(token, "true".split(''))){
    dataReference.data = CreateBooleanData(true);
    i.numberValue = i.numberValue + 1;
  }else if(StringsEqual(token, "false".split(''))){
    dataReference.data = CreateBooleanData(false);
    i.numberValue = i.numberValue + 1;
  }else if(StringsEqual(token, "null".split(''))){
    dataReference.data = CreateNoTypeData();
    i.numberValue = i.numberValue + 1;
  }else if(charIsNumber(token[0]) || token[0] == '-'){
    stringToDecimalResult = nCreateNumberFromDecimalString(token);
    dataReference.data = CreateNumberData(stringToDecimalResult);
    i.numberValue = i.numberValue + 1;
  }else if(token[0] == '\"'){
    substr = strSubstring(token, 1, token.length - 1);
    dataReference.data = CreateStringData(substr);
    i.numberValue = i.numberValue + 1;
  }else{
    str = "".split('');
    str = strConcatenateString(str, "Invalid token first in value: ".split(''));
    str = strAppendString(str, token);
    message.string = str;
    success = false;
  }

  if(success && depth == 0){
    if(StringsEqual(tokens[i.numberValue].string, "<end>".split(''))){
    }else{
      message.string = "The outer value cannot have any tokens following it.".split('');
      success = false;
    }
  }

  return success;
}
function GetJSONObject(tokens, i, depth, dataReference, message){
  var data, value;
  var done, success;
  var key, colon, comma, closeCurly;
  var keystring, str;
  var valueReference;

  data = CreateStructData();
  valueReference = {};
  success = true;
  i.numberValue = i.numberValue + 1;

  if( !StringsEqual(tokens[i.numberValue].string, "}".split('')) ){
    done = false;

    for(;  !done  && success; ){
      key = tokens[i.numberValue].string;

      if(key[0] == '\"'){
        i.numberValue = i.numberValue + 1;
        colon = tokens[i.numberValue].string;
        if(StringsEqual(colon, ":".split(''))){
          i.numberValue = i.numberValue + 1;
          success = GetJSONValueRecursive(tokens, i, depth, valueReference, message);

          if(success){
            keystring = strSubstring(key, 1, key.length - 1);
            value = valueReference.data;

            AddDataToStruct(data.structure, keystring, value);

            comma = tokens[i.numberValue].string;
            if(StringsEqual(comma, ",".split(''))){
              /* OK */
              i.numberValue = i.numberValue + 1;
            }else{
              done = true;
            }
          }
        }else{
          str = "".split('');
          str = strConcatenateString(str, "Expected colon after key in object: ".split(''));
          str = strAppendString(str, colon);
          message.string = str;

          success = false;
          done = true;
        }
      }else{
        message.string = "Expected string as key in object.".split('');

        done = true;
        success = false;
      }
    }
  }

  if(success){
    closeCurly = tokens[i.numberValue].string;

    if(StringsEqual(closeCurly, "}".split(''))){
      /* OK */
      dataReference.data = data;
      i.numberValue = i.numberValue + 1;
    }else{
      message.string = "Expected close curly brackets at end of object value.".split('');
      success = false;
    }
  }

  delete(valueReference);

  return success;
}
function GetJSONArray(tokens, i, depth, dataReference, message){
  var data, value;
  var nextToken, comma;
  var done, success;
  var valueReference;

  i.numberValue = i.numberValue + 1;

  valueReference = {};
  success = true;
  data = CreateArrayData();

  nextToken = tokens[i.numberValue].string;

  if( !StringsEqual(nextToken, "]".split('')) ){
    done = false;
    for(;  !done  && success; ){
      success = GetJSONValueRecursive(tokens, i, depth, valueReference, message);

      if(success){
        value = valueReference.data;
        AddDataToArray(data.array, value);

        comma = tokens[i.numberValue].string;

        if(StringsEqual(comma, ",".split(''))){
          /* OK */
          i.numberValue = i.numberValue + 1;
        }else{
          done = true;
        }
      }
    }
  }

  nextToken = tokens[i.numberValue].string;
  if(StringsEqual(nextToken, "]".split(''))){
    /* OK */
    i.numberValue = i.numberValue + 1;
  }else{
    message.string = "Expected close square bracket at end of array.".split('');
    success = false;
  }

  dataReference.data = data;
  delete(valueReference);

  return success;
}
function JSONTokenize(json, tokensReference, message){
  var i;
  var c;
  var str;
  var stringReference, tokenReference;
  var stringLength;
  var success;
  var ll;

  ll = CreateArray();
  success = true;

  stringLength = {};
  tokenReference = {};

  for(i = 0; i < json.length && success; ){
    c = json[i];

    if(c == '{'){
      AddStringToArray(ll, "{".split(''));
      i = i + 1;
    }else if(c == '}'){
      AddStringToArray(ll, "}".split(''));
      i = i + 1;
    }else if(c == '['){
      AddStringToArray(ll, "[".split(''));
      i = i + 1;
    }else if(c == ']'){
      AddStringToArray(ll, "]".split(''));
      i = i + 1;
    }else if(c == ':'){
      AddStringToArray(ll, ":".split(''));
      i = i + 1;
    }else if(c == ','){
      AddStringToArray(ll, ",".split(''));
      i = i + 1;
    }else if(c == 'f'){
      success = GetJSONPrimitiveName(json, i, message, "false".split(''), tokenReference);
      if(success){
        AddStringToArray(ll, "false".split(''));
        i = i + "false".split('').length;
      }
    }else if(c == 't'){
      success = GetJSONPrimitiveName(json, i, message, "true".split(''), tokenReference);
      if(success){
        AddStringToArray(ll, "true".split(''));
        i = i + "true".split('').length;
      }
    }else if(c == 'n'){
      success = GetJSONPrimitiveName(json, i, message, "null".split(''), tokenReference);
      if(success){
        AddStringToArray(ll, "null".split(''));
        i = i + "null".split('').length;
      }
    }else if(c == ' ' || c == '\n' || c == '\t' || c == '\r'){
      /* Skip. */
      i = i + 1;
    }else if(c == '\"'){
      success = GetJSONString(json, i, tokenReference, stringLength, message);
      if(success){
        AddStringToArray(ll, tokenReference.string);
        i = i + stringLength.numberValue;
      }
    }else if(IsJSONNumberCharacter(c)){
      success = GetJSONNumberToken(json, i, tokenReference, message);
      if(success){
        AddStringToArray(ll, tokenReference.string);
        i = i + tokenReference.string.length;
      }
    }else{
      str = strConcatenateCharacter("Invalid start of Token: ".split(''), c);
      stringReference = CreateStringReference(str);
      message.string = stringReference.string;
      i = i + 1;
      success = false;
    }
  }

  if(success){
    AddStringToArray(ll, "<end>".split(''));
    tokensReference.stringArray = ToStaticStringArray(ll);
    FreeArray(ll);
  }

  return success;
}
function GetJSONNumberToken(json, start, tokenReference, message){
  var c;
  var end, i;
  var done, success;
  var numberString;

  end = json.length;
  done = false;

  for(i = start; i < json.length &&  !done ; i = i + 1){
    c = json[i];
    if( !IsJSONNumberCharacter(c) ){
      done = true;
      end = i;
    }
  }

  numberString = strSubstring(json, start, end);

  success = IsValidJSONNumber(numberString, message);

  tokenReference.string = numberString;

  return success;
}
function IsValidJSONNumber(n, message){
  var success;
  var i;

  i = 0;

  /* JSON allows an optional negative sign. */
  if(n[i] == '-'){
    i = i + 1;
  }

  if(i < n.length){
    success = IsValidJSONNumberAfterSign(n, i, message);
  }else{
    success = false;
    message.string = "Number must contain at least one digit.".split('');
  }

  return success;
}
function IsValidJSONNumberAfterSign(n, i, message){
  var success;

  if(charIsNumber(n[i])){
    /* 0 first means only 0. */
    if(n[i] == '0'){
      i = i + 1;
    }else{
      /* 1-9 first, read following digits. */
      i = IsValidJSONNumberAdvancePastDigits(n, i);
    }

    if(i < n.length){
      success = IsValidJSONNumberFromDotOrExponent(n, i, message);
    }else{
      /* If integer, we are done now. */
      success = true;
    }
  }else{
    success = false;
    message.string = "A number must start with 0-9 (after the optional sign).".split('');
  }

  return success;
}
function IsValidJSONNumberAdvancePastDigits(n, i){
  var done;

  i = i + 1;
  done = false;
  for(; i < n.length &&  !done ; ){
    if(charIsNumber(n[i])){
      i = i + 1;
    }else{
      done = true;
    }
  }

  return i;
}
function IsValidJSONNumberFromDotOrExponent(n, i, message){
  var wasDotAndOrE, success;

  wasDotAndOrE = false;
  success = true;

  if(n[i] == '.'){
    i = i + 1;
    wasDotAndOrE = true;

    if(i < n.length){
      if(charIsNumber(n[i])){
        /* Read digits following decimal point. */
        i = IsValidJSONNumberAdvancePastDigits(n, i);

        if(i == n.length){
          /* If non-scientific decimal number, we are done. */
          success = true;
        }
      }else{
        success = false;
        message.string = "There must be numbers after the decimal point.".split('');
      }
    }else{
      success = false;
      message.string = "There must be numbers after the decimal point.".split('');
    }
  }

  if(i < n.length && success){
    if(n[i] == 'e' || n[i] == 'E'){
      wasDotAndOrE = true;
      success = IsValidJSONNumberFromExponent(n, i, message);
    }else{
      success = false;
      message.string = "Expected e or E.".split('');
    }
  }else if(i == n.length && success){
    /* If number with decimal point. */
    success = true;
  }else{
    success = false;
    message.string = "There must be numbers after the decimal point.".split('');
  }

  if(wasDotAndOrE){
  }else{
    success = false;
    message.string = "Exprected decimal point or e or E.".split('');
  }

  return success;
}
function IsValidJSONNumberFromExponent(n, i, message){
  var success;

  i = i + 1;

  if(i < n.length){
    /* The exponent sign can either + or -, */
    if(n[i] == '+' || n[i] == '-'){
      i = i + 1;
    }

    if(i < n.length){
      if(charIsNumber(n[i])){
        /* Read digits following decimal point. */
        i = IsValidJSONNumberAdvancePastDigits(n, i);

        if(i == n.length){
          /* We found scientific number. */
          success = true;
        }else{
          success = false;
          message.string = "There was characters following the exponent.".split('');
        }
      }else{
        success = false;
        message.string = "There must be a digit following the optional exponent sign.".split('');
      }
    }else{
      success = false;
      message.string = "There must be a digit following optional the exponent sign.".split('');
    }
  }else{
    success = false;
    message.string = "There must be a sign or a digit following e or E.".split('');
  }

  return success;
}
function IsJSONNumberCharacter(c){
  var numericCharacters;
  var found;
  var i;

  numericCharacters = "0123456789.-+eE".split('');

  found = false;

  for(i = 0; i < numericCharacters.length; i = i + 1){
    if(numericCharacters[i] == c){
      found = true;
    }
  }

  return found;
}
function GetJSONPrimitiveName(string, start, message, primitive, tokenReference){
  var c, p;
  var done, success;
  var i;
  var str, token;

  done = false;
  success = true;

  token = "".split('');

  for(i = start; i < string.length && ((i - start) < primitive.length) &&  !done ; i = i + 1){
    c = string[i];
    p = primitive[i - start];
    if(c == p){
      /* OK */
      if((i + 1 - start) == primitive.length){
        done = true;
      }
    }else{
      str = "".split('');
      str = strConcatenateString(str, "Primitive invalid: ".split(''));
      str = strAppendCharacter(str, c);
      str = strAppendString(str, " vs ".split(''));
      str = strAppendCharacter(str, p);

      message.string = str;
      done = true;
      success = false;
    }
  }

  if(done){
    if(StringsEqual(primitive, "false".split(''))){
      token = "false".split('');
    }
    if(StringsEqual(primitive, "true".split(''))){
      token = "true".split('');
    }
    if(StringsEqual(primitive, "null".split(''))){
      token = "null".split('');
    }
  }else{
    message.string = "Primitive invalid".split('');
    success = false;
  }

  tokenReference.string = token;

  return success;
}
function GetJSONString(json, start, tokenReference, stringLengthReference, message){
  var success, done;
  var string, hex;
  var characterCount, hexReference;
  var i, l, c;
  var errorMessage;

  characterCount = CreateNumberReference(0);
  hex = CreateString(4, '0');
  hexReference = {};
  errorMessage = {};

  success = IsValidJSONStringInJSON(json, start, characterCount, stringLengthReference, message);

  if(success){
    l = characterCount.numberValue;
    string = [];
    string.length = l;

    c = 0;
    string[c] = '\"';
    c = c + 1;

    done = false;
    for(i = start + 1;  !done ; i = i + 1){
      if(json[i] == '\\'){
        i = i + 1;
        if(json[i] == '\"' || json[i] == '\\' || json[i] == '/'){
          string[c] = json[i];
          c = c + 1;
        }else if(json[i] == 'b'){
          string[c] = String.fromCharCode(8);
          c = c + 1;
        }else if(json[i] == 'f'){
          string[c] = String.fromCharCode(12);
          c = c + 1;
        }else if(json[i] == 'n'){
          string[c] = String.fromCharCode(10);
          c = c + 1;
        }else if(json[i] == 'r'){
          string[c] = String.fromCharCode(13);
          c = c + 1;
        }else if(json[i] == 't'){
          string[c] = String.fromCharCode(9);
          c = c + 1;
        }else if(json[i] == 'u'){
          i = i + 1;
          hex[0] = charToUpperCase(json[i + 0]);
          hex[1] = charToUpperCase(json[i + 1]);
          hex[2] = charToUpperCase(json[i + 2]);
          hex[3] = charToUpperCase(json[i + 3]);
          nCreateNumberFromStringWithCheck(hex, 16, hexReference, errorMessage);
          string[c] = String.fromCharCode(hexReference.numberValue);
          i = i + 3;
          c = c + 1;
        }
      }else if(json[i] == '\"'){
        string[c] = json[i];
        c = c + 1;
        done = true;
      }else{
        string[c] = json[i];
        c = c + 1;
      }
    }

    tokenReference.string = string;
    success = true;
  }else{
    message.string = "End of string was not found.".split('');
    success = false;
  }

  return success;
}
function IsValidJSONString(jsonString, message){
  var valid;
  var numberReference, stringLength;

  numberReference = {};
  stringLength = {};

  valid = IsValidJSONStringInJSON(jsonString, 0, numberReference, stringLength, message);

  return valid;
}
function IsValidJSONStringInJSON(json, start, characterCount, stringLengthReference, message){
  var success, done;
  var i, j;
  var c;

  success = true;
  done = false;

  characterCount.numberValue = 1;

  for(i = start + 1; i < json.length &&  !done  && success; i = i + 1){
    if( !IsJSONIllegalControllCharacter(json[i]) ){
      if(json[i] == '\\'){
        i = i + 1;
        if(i < json.length){
          if(json[i] == '\"' || json[i] == '\\' || json[i] == '/' || json[i] == 'b' || json[i] == 'f' || json[i] == 'n' || json[i] == 'r' || json[i] == 't'){
            characterCount.numberValue = characterCount.numberValue + 1;
          }else if(json[i] == 'u'){
            if(i + 4 < json.length){
              for(j = 0; j < 4 && success; j = j + 1){
                c = json[i + j + 1];
                if(nCharacterIsNumberCharacterInBase(c, 16) || c == 'a' || c == 'b' || c == 'c' || c == 'd' || c == 'e' || c == 'f'){
                }else{
                  success = false;
                  message.string = "\\u must be followed by four hexadecimal digits.".split('');
                }
              }
              characterCount.numberValue = characterCount.numberValue + 1;
              i = i + 4;
            }else{
              success = false;
              message.string = "\\u must be followed by four characters.".split('');
            }
          }else{
            success = false;
            message.string = "Escaped character invalid.".split('');
          }
        }else{
          success = false;
          message.string = "There must be at least two character after string escape.".split('');
        }
      }else if(json[i] == '\"'){
        characterCount.numberValue = characterCount.numberValue + 1;
        done = true;
      }else{
        characterCount.numberValue = characterCount.numberValue + 1;
      }
    }else{
      success = false;
      message.string = "Unicode code points 0-31 not allowed in JSON string.".split('');
    }
  }

  if(done){
    stringLengthReference.numberValue = i - start;
  }else{
    success = false;
    message.string = "String must end with \".".split('');
  }

  return success;
}
function IsJSONIllegalControllCharacter(c){
  var cnr;
  var isControll;

  cnr = c.charCodeAt(0);

  if(cnr >= 0 && cnr < 32){
    isControll = true;
  }else{
    isControll = false;
  }

  return isControll;
}
function IsValidJSON(json, message){
  var success;
  var elementReference;

  elementReference = {};

  success = ReadJSON(json, elementReference, message);

  if(success){
    FreeData(elementReference.data);
  }

  return success;
}
function ComputeJSONStringLength(data){
  var result;

  result = 0;

  if(IsStructure(data)){
    result = result + ComputeJSONObjectStringLength(data);
  }else if(IsString(data)){
    result = JSONEscapedStringLength(data.string) + 2;
  }else if(IsArray(data)){
    result = result + ComputeJSONArrayStringLength(data);
  }else if(IsNumber(data)){
    result = result + ComputeJSONNumberStringLength(data);
  }else if(IsNoType(data)){
    result = result + "null".split('').length;
  }else if(IsBoolean(data)){
    result = result + ComputeJSONBooleanStringLength(data);
  }

  return result;
}
function ComputeJSONBooleanStringLength(data){
  var result;

  if(data.booleanx){
    result = "true".split('').length;
  }else{
    result = "false".split('').length;
  }

  return result;
}
function ComputeJSONNumberStringLength(data){
  var lengthx;
  var a;

  if(data.number != 0){
    if(Math.abs(data.number) >= Math.pow(10, 15) || Math.abs(data.number) <= Math.pow(10,  -15)){
      a = nCreateStringScientificNotationDecimalFromNumber(data.number);
      lengthx = a.length;
    }else{
      a = nCreateStringDecimalFromNumber(data.number);
      lengthx = a.length;
    }
  }else{
    lengthx = 1;
  }

  return lengthx;
}
function ComputeJSONArrayStringLength(data){
  var arrayElement;
  var i;
  var lengthx;

  lengthx = 1;

  for(i = 0; i < data.array.lengthx; i = i + 1){
    arrayElement = ArrayIndex(data.array, i);

    lengthx = lengthx + ComputeJSONStringLength(arrayElement);

    if(i + 1 != data.array.lengthx){
      lengthx = lengthx + 1;
    }
  }

  lengthx = lengthx + 1;

  return lengthx;
}
function ComputeJSONObjectStringLength(data){
  var key;
  var i;
  var keys;
  var objectElement;
  var lengthx;

  lengthx = 1;

  keys = GetStructKeys(data.structure);
  for(i = 0; i < keys.length; i = i + 1){
    key = keys[i].string;
    objectElement = GetDataFromStruct(data.structure, key);

    lengthx = lengthx + 1;
    lengthx = lengthx + JSONEscapedStringLength(key);
    lengthx = lengthx + 1;
    lengthx = lengthx + 1;

    lengthx = lengthx + ComputeJSONStringLength(objectElement);

    if(i + 1 != keys.length){
      lengthx = lengthx + 1;
    }
  }

  lengthx = lengthx + 1;

  return lengthx;
}
function UnicodeToUTF8(unicode){
  var utf8;
  var lengthx, i, pos, p1, p2, p3, p4, code;

  lengthx = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    code = unicode[i];

    lengthx = lengthx + 1;
    if(code >= 128){
      lengthx = lengthx + 1;
    }
    if(code >= 2048){
      lengthx = lengthx + 1;
    }
    if(code >= 65536){
      lengthx = lengthx + 1;
    }
  }

  utf8 = CreateByteArray(lengthx);

  pos = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    code = unicode[i];

    if(code < 128){
      SetByte(utf8, pos, code);
      pos = pos + 1;
    }else if(code < 2048){
      p1 = code%64;
      p2 = Math.floor(code/64)%64;

      SetByte(utf8, pos, 128 + 64 + p2);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p1);
      pos = pos + 1;
    }else if(code < 65536){
      p1 = code%64;
      p2 = Math.floor(code/64)%64;
      p3 = Math.floor(code/4096);

      SetByte(utf8, pos, 128 + 64 + 32 + p3);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p2);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p1);
      pos = pos + 1;
    }else{
      p1 = code%64;
      p2 = Math.floor(code/64)%64;
      p3 = Math.floor(code/4096)%64;
      p4 = Math.floor(code/262144);

      SetByte(utf8, pos, 128 + 64 + 32 + 16 + p4);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p3);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p2);
      pos = pos + 1;
      SetByte(utf8, pos, 128 + p1);
      pos = pos + 1;
    }
  }

  return utf8;
}
function UTF8ToUnicode(utf8){
  var utf;
  var lengthx, i, pos, p1, p2, p3, p4, code;

  lengthx = 0;
  for(i = 0; i < ByteArrayLength(utf8); ){
    code = Math.floor(GetByte(utf8, i)/8)*8;

    if(code < 128){
      lengthx = lengthx + 1;
      i = i + 1;
    }else if(code == 128 + 64){
      lengthx = lengthx + 1;
      i = i + 2;
    }else if(code == 128 + 64 + 32){
      lengthx = lengthx + 1;
      i = i + 3;
    }else if(code == 128 + 64 + 32 + 16){
      lengthx = lengthx + 1;
      i = i + 4;
    }
  }

  utf = [];
  utf.length = lengthx;

  pos = 0;
  for(i = 0; i < ByteArrayLength(utf8); ){
    code = Math.floor(GetByte(utf8, i)/8)*8;

    if(code < 128){
      utf[pos] = GetByte(utf8, i);
      i = i + 1;
      pos = pos + 1;
    }else if(code == 128 + 64){
      p1 = GetByte(utf8, i + 1) - 128;
      p2 = GetByte(utf8, i) - 128 - 64;

      utf[pos] = p2*64 + p1;

      i = i + 2;
      pos = pos + 1;
    }else if(code == 128 + 64 + 32){
      p1 = GetByte(utf8, i + 2) - 128;
      p2 = GetByte(utf8, i + 1) - 128;
      p3 = GetByte(utf8, i) - 128 - 64 - 32;

      utf[pos] = p3*4096 + p2*64 + p1;

      i = i + 3;
      pos = pos + 1;
    }else if(code == 128 + 64 + 32 + 16){
      p1 = GetByte(utf8, i + 3) - 128;
      p2 = GetByte(utf8, i + 2) - 128;
      p3 = GetByte(utf8, i + 1) - 128;
      p4 = GetByte(utf8, i) - 128 - 64 - 32 - 16;

      utf[pos] = p4*262144 + p3*4096 + p2*64 + p1;

      i = i + 4;
      pos = pos + 1;
    }
  }

  return utf;
}
function UTF16ToUnicode(utf16){
  var unicode;
  var lengthx, i, pos, hs, ls, code;
  var surrogateLowerBound, surrogateUpperBound, highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  surrogateLowerBound = 55296;
  surrogateUpperBound = 57344;
  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < utf16.length; ){
    code = utf16[i].charCodeAt(0);

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      lengthx = lengthx + 1;
      i = i + 1;
    }else{
      lengthx = lengthx + 1;
      i = i + 2;
    }
  }

  unicode = [];
  unicode.length = lengthx;

  pos = 0;
  for(i = 0; i < utf16.length; ){
    code = utf16[i].charCodeAt(0);

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      unicode[pos] = utf16[i].charCodeAt(0);
      i = i + 1;
      pos = pos + 1;
    }else{
      hs = utf16[i].charCodeAt(0);
      ls = utf16[i + 1].charCodeAt(0);

      unicode[pos] = (hs - highSurrogateBase)*highMultiplier + (ls - lowSurrogateBase) + surrogateBase;

      i = i + 2;
      pos = pos + 1;
    }
  }

  return unicode;
}
function UnicodeToUTF16(unicode){
  var utf16;
  var lengthx, code, i, pos, hs, ls;
  var highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      lengthx = lengthx + 1;
    }else{
      lengthx = lengthx + 2;
    }
  }

  utf16 = [];
  utf16.length = lengthx;

  pos = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      utf16[pos] = String.fromCharCode(unicode[i]);
      pos = pos + 1;
    }else{
      code = unicode[i] - surrogateBase;
      hs = Math.floor(code/highMultiplier) + highSurrogateBase;
      ls = code%highMultiplier + lowSurrogateBase;

      utf16[pos] = String.fromCharCode(hs);
      pos = pos + 1;
      utf16[pos] = String.fromCharCode(ls);
      pos = pos + 1;
    }
  }

  return utf16;
}
function UTF8ToUTF16(utf8){
  var unicode;
  var utf16;

  unicode = UTF8ToUnicode(utf8);
  utf16 = UnicodeToUTF16(unicode);

  delete(unicode);

  return utf16;
}
function UTF16ToUTF8(utf16){
  var unicode;
  var utf8;

  unicode = UTF16ToUnicode(utf16);
  utf8 = UnicodeToUTF8(unicode);

  delete(unicode);

  return utf8;
}
function IsValidUnicode(unicode){
  var lower, upper, i, code;
  var valid;

  lower = 0;
  upper = 1114112;

  valid = true;
  for(i = 0; i < unicode.length && valid; i = i + 1){
    code = unicode[i];

    if(code == Math.floor(code)){
      if(code >= lower && code < upper){
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function IsValidUtf16(utf16){
  var i, code;
  var valid;
  var surrogateLowerBound, surrogateUpperBound;

  surrogateLowerBound = 55296;
  surrogateUpperBound = 57344;

  valid = true;
  for(i = 0; i < utf16.length && valid; i = i + 1){
    code = utf16[i].charCodeAt(0);

    if(code == Math.floor(code)){
      if(code < surrogateLowerBound || code >= surrogateUpperBound){
      }else if(i + 1 < utf16.length){
        i = i + 1;
        code = utf16[i].charCodeAt(0);

        if(code < surrogateLowerBound || code >= surrogateUpperBound){
          valid = false;
        }
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function IsValidUtf8(utf8){
  var i, code1, code2, code3, code4;
  var valid;

  valid = true;
  for(i = 0; i < ByteArrayLength(utf8) && valid; i = i + 1){
    code1 = GetByte(utf8, i);

    if(code1 == Math.floor(code1)){
      if(code1 < 128){
      }else if(code1 < 128 + 64){
        valid = false;
      }else if(code1 < 128 + 64 + 32){
        if(i + 1 < ByteArrayLength(utf8)){
          i = i + 1;
          code2 = GetByte(utf8, i);

          if(code2 >= 128 && code2 < 128 + 64){
          }else{
            valid = false;
          }
        }else{
          valid = false;
        }
      }else if(code1 < 128 + 64 + 32 + 16){
        if(i + 2 < ByteArrayLength(utf8)){
          i = i + 1;
          code2 = GetByte(utf8, i);
          i = i + 1;
          code3 = GetByte(utf8, i);

          if(code2 >= 128 && code2 < 128 + 64 && code3 >= 128 && code3 < 128 + 64){
          }else{
            valid = false;
          }
        }else{
          valid = false;
        }
      }else if(i + 3 < ByteArrayLength(utf8)){
        i = i + 1;
        code2 = GetByte(utf8, i);
        i = i + 1;
        code3 = GetByte(utf8, i);
        i = i + 1;
        code4 = GetByte(utf8, i);

        if(code2 >= 128 && code2 < 128 + 64 && code3 >= 128 && code3 < 128 + 64 && code4 >= 128 && code4 < 128 + 64){
        }else{
          valid = false;
        }
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function CreateArrayData(){
  var data;

  data = {};
  data.isArray = true;
  data.array = CreateArray();

  return data;
}
function CreateStructData(){
  var data;

  data = {};
  data.isStruture = true;
  data.structure = CreateStructure();

  return data;
}
function CreateStructure(){
  var st;

  st = {};
  st.keys = CreateArray();
  st.values = CreateArray();

  return st;
}
function CreateNumberData(n){
  var data;

  data = {};
  data.isNumber = true;
  data.number = n;

  return data;
}
function CreateBooleanData(b){
  var data;

  data = {};
  data.isBoolean = true;
  data.booleanx = b;

  return data;
}
function CreateStringData(string){
  var data;

  data = {};
  data.isString = true;
  data.string = string;

  return data;
}
function CreateNoTypeData(){
  var data;

  data = {};
  data.isStruture = false;
  data.isArray = false;
  data.isNumber = false;
  data.isBoolean = false;
  data.isString = false;

  return data;
}
function AddStructToArray(ar, st){
  var data;

  data = CreateStructData();
  delete(data.structure);
  data.structure = st;

  ArrayAdd(ar, data);
}
function AddArrayToArray(ar, ar2){
  var data;

  data = CreateArrayData();
  delete(data.array);
  data.array = ar2;

  ArrayAdd(ar, data);
}
function AddNumberToArray(ar, n){
  ArrayAdd(ar, CreateNumberData(n));
}
function AddBooleanToArray(ar, b){
  ArrayAdd(ar, CreateBooleanData(b));
}
function AddStringToArray(ar, str){
  ArrayAdd(ar, CreateStringData(str));
}
function AddDataToArray(ar, data){
  ArrayAdd(ar, data);
}
function StructKeys(st){
  return ArrayLength(st.keys);
}
function StructHasKey(st, key){
  var i;
  var hasKey;

  hasKey = false;
  for(i = 0; i < StructKeys(st); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      hasKey = true;
    }
  }

  return hasKey;
}
function StructKeyIndex(st, key){
  var i;
  var index;

  index =  -1;
  for(i = 0; i < StructKeys(st); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      index = i;
    }
  }

  return index;
}
function GetStructKeys(st){
  var keys;
  var nr, i;

  nr = StructKeys(st);

  keys = [];
  keys.length = nr;

  for(i = 0; i < nr; i = i + 1){
    keys[i] = {};
    keys[i].string = CopyString(st.keys.array[i].string);
  }

  return keys;
}
function GetStructFromStruct(st, key){
  var i;
  var r;

  r = {};
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      r = st.values.array[i].structure;
    }
  }

  return r;
}
function GetArrayFromStruct(st, key){
  var i;
  var r;

  r = {};
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      r = st.values.array[i].array;
    }
  }

  return r;
}
function GetNumberFromStruct(st, key){
  var i, r;

  r = 0;
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      r = st.values.array[i].number;
    }
  }

  return r;
}
function GetBooleanFromStruct(st, key){
  var i;
  var r;

  r = false;
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      r = st.values.array[i].booleanx;
    }
  }

  return r;
}
function GetStringFromStruct(st, key){
  var i;
  var r;

  r = "".split('');
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      r = st.values.array[i].string;
    }
  }

  return r;
}
function GetDataFromStruct(st, key){
  var i;
  var r;

  r = {};
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      delete(r);
      r = st.values.array[i];
    }
  }

  return r;
}
function GetDataFromStructWithCheck(st, key, foundRef){
  var i;
  var r;

  r = {};
  foundRef.booleanValue = false;
  for(i = 0; i < ArrayLength(st.keys); i = i + 1){
    if(StringsEqual(st.keys.array[i].string, key)){
      delete(r);
      foundRef.booleanValue = true;
      r = st.values.array[i];
    }
  }

  return r;
}
function AddStructToStruct(st, key, struct){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    delete(st.values.array[i].structure);
    st.values.array[i].structure = struct;
  }else{
    AddStringToArray(st.keys, key);
    AddStructToArray(st.values, struct);
  }
}
function AddArrayToStruct(st, key, ar){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    delete(st.values.array[i].array);
    st.values.array[i].array = ar;
  }else{
    AddStringToArray(st.keys, key);
    AddArrayToArray(st.values, ar);
  }
}
function AddNumberToStruct(st, key, n){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    st.values.array[i].number = n;
  }else{
    AddStringToArray(st.keys, key);
    AddNumberToArray(st.values, n);
  }
}
function AddBooleanToStruct(st, key, b){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    st.values.array[i].booleanx = b;
  }else{
    AddStringToArray(st.keys, key);
    AddBooleanToArray(st.values, b);
  }
}
function AddStringToStruct(st, key, value){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    delete(st.values.array[i].string);
    st.values.array[i].string = value;
  }else{
    AddStringToArray(st.keys, key);
    AddStringToArray(st.values, value);
  }
}
function AddDataToStruct(st, key, data){
  var i;

  if(StructHasKey(st, key)){
    i = StructKeyIndex(st, key);
    FreeData(st.values.array[i]);
    st.values.array[i] = data;
  }else{
    AddStringToArray(st.keys, key);
    AddDataToArray(st.values, data);
  }
}
function FreeData(data){
  var i;
  var st;

  if(data.isStruture){
    st = data.structure;
    for(i = 0; i < StructKeys(st); i = i + 1){
      FreeData(ArrayIndex(st.keys, i));
      FreeData(ArrayIndex(st.values, i));
    }
    delete(st);
  }else if(data.isArray){
    FreeArray(data.array);
  }

  delete(data);
}
function FreeArray(array){
  var i;

  for(i = 0; i < ArrayLength(array); i = i + 1){
    FreeData(array.array[i]);
  }

  delete(array.array);
  delete(array);
}
function DataTypeEquals(a, b){
  var equal;

  equal = true;
  equal = equal && a.isStruture == b.isStruture;
  equal = equal && a.isArray == b.isArray;
  equal = equal && a.isNumber == b.isNumber;
  equal = equal && a.isBoolean == b.isBoolean;
  equal = equal && a.isString == b.isString;

  return equal;
}
function IsStructure(a){
  var isStructure;

  isStructure = a.isStruture;
  if(a.isArray || a.isNumber || a.isBoolean || a.isString){
    isStructure = false;
  }

  return isStructure;
}
function IsArray(a){
  var isArray;

  isArray = a.isArray;
  if(a.isStruture || a.isNumber || a.isBoolean || a.isString){
    isArray = false;
  }

  return isArray;
}
function IsNumber(a){
  var isNumber;

  isNumber = a.isNumber;
  if(a.isStruture || a.isArray || a.isBoolean || a.isString){
    isNumber = false;
  }

  return isNumber;
}
function IsBoolean(a){
  var isBoolean;

  isBoolean = a.isBoolean;
  if(a.isStruture || a.isArray || a.isNumber || a.isString){
    isBoolean = false;
  }

  return isBoolean;
}
function IsString(a){
  var isString;

  isString = a.isString;
  if(a.isStruture || a.isArray || a.isNumber || a.isBoolean){
    isString = false;
  }

  return isString;
}
function IsNoType(a){
  var isNoType;

  if( !a.isString  &&  !a.isStruture  &&  !a.isArray  &&  !a.isNumber  &&  !a.isBoolean ){
    isNoType = true;
  }else{
    isNoType = false;
  }

  return isNoType;
}
function CreateArray(){
  var array;

  array = {};
  array.array = [];
  array.array.length = 10;
  array.lengthx = 0;

  return array;
}
function CreateArrayWithInitialCapacity(capacity){
  var array;

  array = {};
  array.array = [];
  array.array.length = capacity;
  array.lengthx = 0;

  return array;
}
function ArrayAdd(array, value){
  if(array.lengthx == array.array.length){
    ArrayIncreaseSize(array);
  }

  array.array[array.lengthx] = value;
  array.lengthx = array.lengthx + 1;
}
function ArrayIncreaseSize(array){
  var newLength, i;
  var newArray;

  newLength = Math.round(array.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < array.array.length; i = i + 1){
    newArray[i] = array.array[i];
  }

  delete(array.array);

  array.array = newArray;
}
function ArrayDecreaseSizeNecessary(array){
  var needsDecrease;

  needsDecrease = false;

  if(array.lengthx > 10){
    needsDecrease = array.lengthx <= Math.round(array.array.length*2/3);
  }

  return needsDecrease;
}
function ArrayDecreaseSize(array){
  var newLength, i;
  var newArray;

  newLength = Math.round(array.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = array.array[i];
  }

  delete(array.array);

  array.array = newArray;
}
function ArrayIndex(array, index){
  return array.array[index];
}
function ArrayLength(array){
  return array.lengthx;
}
function ArrayInsert(array, index, value){
  var i;

  if(array.lengthx == array.array.length){
    ArrayIncreaseSize(array);
  }

  for(i = array.lengthx; i > index; i = i - 1){
    array.array[i] = array.array[i - 1];
  }

  array.array[index] = value;

  array.lengthx = array.lengthx + 1;
}
function ArraySet(array, index, value){
  var success;

  if(index < array.lengthx){
    array.array[index] = value;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function ArrayRemove(array, index){
  var i;

  for(i = index; i < array.lengthx - 1; i = i + 1){
    array.array[i] = array.array[i + 1];
  }

  array.lengthx = array.lengthx - 1;

  if(ArrayDecreaseSizeNecessary(array)){
    ArrayDecreaseSize(array);
  }
}
function ToStaticArray(arc){
  var array;
  var i;

  array = [];
  array.length = arc.lengthx;

  for(i = 0; i < arc.lengthx; i = i + 1){
    array[i] = arc.array[i];
  }

  return array;
}
function ToStaticNumberArray(array){
  var result;
  var i, n;

  n = ArrayLength(array);

  result = [];
  result.length = n;

  for(i = 0; i < n; i = i + 1){
    result[i] = ArrayIndex(array, i).number;
  }

  return result;
}
function ToStaticBooleanArray(array){
  var result;
  var i, n;

  n = ArrayLength(array);

  result = [];
  result.length = n;

  for(i = 0; i < n; i = i + 1){
    result[i] = ArrayIndex(array, i).booleanx;
  }

  return result;
}
function ToStaticStringArray(array){
  var result;
  var i, n;

  n = ArrayLength(array);

  result = [];
  result.length = n;

  for(i = 0; i < n; i = i + 1){
    result[i] = {};
    result[i].string = ArrayIndex(array, i).string;
  }

  return result;
}
function ToStaticArrayArray(array){
  var result;
  var i, n;

  n = ArrayLength(array);

  result = [];
  result.length = n;

  for(i = 0; i < n; i = i + 1){
    result[i] = ArrayIndex(array, i).array;
  }

  return result;
}
function ToStaticStructArray(array){
  var result;
  var i, n;

  n = ArrayLength(array);

  result = [];
  result.length = n;

  for(i = 0; i < n; i = i + 1){
    result[i] = ArrayIndex(array, i).structure;
  }

  return result;
}
function StaticArrayToArrayWithOptimalSize(src){
  var dst;
  var i;
  var c, n, newCapacity;

  /*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
         */

  c = src.length;
  n = (Math.log(c) - 1)/Math.log(3/2);

  newCapacity = Math.ceil(10*Math.pow(3/2, Math.ceil(n)));

  dst = CreateArrayWithInitialCapacity(newCapacity);

  for(i = 0; i < src.length; i = i + 1){
    dst.array[i] = src[i];
  }

  return dst;
}
function StaticArrayToArray(src){
  var i;
  var dst;

  dst = CreateArrayWithInitialCapacity(src.length);
  for(i = 0; i < src.length; i = i + 1){
    dst.array[i] = src[i];
  }
  dst.lengthx = src.length;

  return dst;
}
function ArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DataArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function CreateBooleanReference(value){
  var ref;

  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;

  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;

  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;

  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;

  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;

  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;

  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;

  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !StringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function strWriteStringToStingStream(stream, index, src){
  var i;

  for(i = 0; i < src.length; i = i + 1){
    stream[index.numberValue + i] = src[i];
  }
  index.numberValue = index.numberValue + src.length;
}
function strWriteCharacterToStingStream(stream, index, src){
  stream[index.numberValue] = src;
  index.numberValue = index.numberValue + 1;
}
function strWriteBooleanToStingStream(stream, index, src){
  if(src){
    strWriteStringToStingStream(stream, index, "true".split(''));
  }else{
    strWriteStringToStingStream(stream, index, "false".split(''));
  }
}
function strSubstringWithCheck(string, from, to, stringReference){
  var success;

  if(from >= 0 && from <= string.length && to >= 0 && to <= string.length && from <= to){
    stringReference.string = strSubstring(string, from, to);
    success = true;
  }else{
    success = false;
  }

  return success;
}
function strSubstring(string, from, to){
  var n;
  var i, lengthx;

  lengthx = to - from;

  n = [];
  n.length = lengthx;

  for(i = from; i < to; i = i + 1){
    n[i - from] = string[i];
  }

  return n;
}
function strAppendString(s1, s2){
  var newString;

  newString = strConcatenateString(s1, s2);

  delete(s1);

  return newString;
}
function strConcatenateString(s1, s2){
  var newString;
  var i;

  newString = [];
  newString.length = s1.length + s2.length;

  for(i = 0; i < s1.length; i = i + 1){
    newString[i] = s1[i];
  }

  for(i = 0; i < s2.length; i = i + 1){
    newString[s1.length + i] = s2[i];
  }

  return newString;
}
function strAppendCharacter(string, c){
  var newString;

  newString = strConcatenateCharacter(string, c);

  delete(string);

  return newString;
}
function strConcatenateCharacter(string, c){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + 1;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  newString[string.length] = c;

  return newString;
}
function strSplitByCharacter(toSplit, splitBy){
  var split;
  var stringToSplitBy;

  stringToSplitBy = [];
  stringToSplitBy.length = 1;
  stringToSplitBy[0] = splitBy;

  split = strSplitByString(toSplit, stringToSplitBy);

  delete(stringToSplitBy);

  return split;
}
function strIndexOfCharacter(string, character, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function strSubstringEqualsWithCheck(string, from, substring, equalsReference){
  var success;

  if(from < string.length){
    success = true;
    equalsReference.booleanValue = strSubstringEquals(string, from, substring);
  }else{
    success = false;
  }

  return success;
}
function strSubstringEquals(string, from, substring){
  var i;
  var equal;

  equal = true;
  if(string.length - from >= substring.length){
    for(i = 0; i < substring.length && equal; i = i + 1){
      if(string[from + i] != substring[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function strIndexOfString(string, substring, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length - substring.length + 1 &&  !found ; i = i + 1){
    if(strSubstringEquals(string, i, substring)){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function strContainsCharacter(string, character){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
    }
  }

  return found;
}
function strContainsString(string, substring){
  return strIndexOfString(string, substring, {});
}
function strToUpperCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToUpperCase(string[i]);
  }
}
function strToLowerCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToLowerCase(string[i]);
  }
}
function strEqualsIgnoreCase(a, b){
  var equal;
  var i;

  if(a.length == b.length){
    equal = true;
    for(i = 0; i < a.length && equal; i = i + 1){
      if(charToLowerCase(a[i]) != charToLowerCase(b[i])){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function strReplaceString(string, toReplace, replaceWith){
  var result;
  var i, j;
  var equalsReference;
  var success;
  var da;

  da = CreateDynamicArrayCharacters();

  equalsReference = {};

  for(i = 0; i < string.length; ){
    success = strSubstringEqualsWithCheck(string, i, toReplace, equalsReference);
    if(success){
      success = equalsReference.booleanValue;
    }

    if(success && toReplace.length > 0){
      for(j = 0; j < replaceWith.length; j = j + 1){
        DynamicArrayAddCharacter(da, replaceWith[j]);
      }
      i = i + toReplace.length;
    }else{
      DynamicArrayAddCharacter(da, string[i]);
      i = i + 1;
    }
  }

  result = DynamicArrayCharactersToArray(da);

  FreeDynamicArrayCharacters(da);

  return result;
}
function strReplaceCharacterToNew(string, toReplace, replaceWith){
  var result;
  var i;

  result = [];
  result.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      result[i] = replaceWith;
    }else{
      result[i] = string[i];
    }
  }

  return result;
}
function strReplaceCharacter(string, toReplace, replaceWith){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      string[i] = replaceWith;
    }
  }
}
function strTrim(string){
  var result;
  var i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
  var firstNonWhitespaceFound;

  /* Find whitepaces at the start. */
  lastWhitespaceLocationStart =  -1;
  firstNonWhitespaceFound = false;
  for(i = 0; i < string.length &&  !firstNonWhitespaceFound ; i = i + 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationStart = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  /* Find whitepaces at the end. */
  lastWhitespaceLocationEnd = string.length;
  firstNonWhitespaceFound = false;
  for(i = string.length - 1; i >= 0 &&  !firstNonWhitespaceFound ; i = i - 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationEnd = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
    result = strSubstring(string, lastWhitespaceLocationStart + 1, lastWhitespaceLocationEnd);
  }else{
    result = [];
    result.length = 0;
  }

  return result;
}
function strStartsWith(string, start){
  var startsWithString;

  startsWithString = false;
  if(string.length >= start.length){
    startsWithString = strSubstringEquals(string, 0, start);
  }

  return startsWithString;
}
function strEndsWith(string, end){
  var endsWithString;

  endsWithString = false;
  if(string.length >= end.length){
    endsWithString = strSubstringEquals(string, string.length - end.length, end);
  }

  return endsWithString;
}
function strSplitByString(toSplit, splitBy){
  var split;
  var next;
  var i;
  var c;
  var n;

  split = [];
  split.length = 0;

  next = [];
  next.length = 0;
  for(i = 0; i < toSplit.length; ){
    c = toSplit[i];

    if(strSubstringEquals(toSplit, i, splitBy)){
      n = {};
      n.string = next;
      split = AddString(split, n);
      next = [];
      next.length = 0;
      i = i + splitBy.length;
    }else{
      next = strAppendCharacter(next, c);
      i = i + 1;
    }
  }

  n = {};
  n.string = next;
  split = AddString(split, n);

  return split;
}
function strStringIsBefore(a, b){
  var before, equal, done;
  var i;

  before = false;
  equal = true;
  done = false;

  if(a.length == 0 && b.length > 0){
    before = true;
  }else{
    for(i = 0; i < a.length && i < b.length &&  !done ; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
      if(charCharacterIsBefore(a[i], b[i])){
        before = true;
      }
      if(charCharacterIsBefore(b[i], a[i])){
        done = true;
      }
    }

    if(equal){
      if(a.length < b.length){
        before = true;
      }
    }
  }

  return before;
}
function strJoinStringsWithSeparator(strings, separator){
  var result, string;
  var lengthx, i;
  var index;

  index = CreateNumberReference(0);

  lengthx = 0;
  for(i = 0; i < strings.length; i = i + 1){
    lengthx = lengthx + strings[i].string.length;
  }
  lengthx = lengthx + (strings.length - 1)*separator.length;

  result = [];
  result.length = lengthx;

  for(i = 0; i < strings.length; i = i + 1){
    string = strings[i].string;
    strWriteStringToStingStream(result, index, string);
    if(i + 1 < strings.length){
      strWriteStringToStingStream(result, index, separator);
    }
  }

  delete(index);

  return result;
}
function strJoinStrings(strings){
  var result, string;
  var lengthx, i;
  var index;

  index = CreateNumberReference(0);

  lengthx = 0;
  for(i = 0; i < strings.length; i = i + 1){
    lengthx = lengthx + strings[i].string.length;
  }

  result = [];
  result.length = lengthx;

  for(i = 0; i < strings.length; i = i + 1){
    string = strings[i].string;
    strWriteStringToStingStream(result, index, string);
  }

  delete(index);

  return result;
}
function nCreateStringScientificNotationDecimalFromNumber(n){
  var mantissaReference, exponentReference;
  var e;
  var isPositive;
  var result;

  mantissaReference = {};
  exponentReference = {};
  result = [];
  result.length = 0;

  if(n < 0){
    isPositive = false;
    n =  -n;
  }else{
    isPositive = true;
  }

  if(n == 0){
    e = 0;
  }else{
    e = nGetFirstDecimalDigitPosition(n);

    if(e < 0){
      n = n*Math.pow(10, Math.abs(e));
    }else{
      n = n/Math.pow(10, e);
    }
  }

  mantissaReference.string = nCreateStringDecimalFromNumber(n);
  exponentReference.string = nCreateStringDecimalFromNumber(e);

  if( !isPositive ){
    result = strAppendString(result, "-".split(''));
  }

  result = strAppendString(result, mantissaReference.string);
  result = strAppendString(result, "e".split(''));
  result = strAppendString(result, exponentReference.string);

  return result;
}
function nCreateStringDecimalFromNumber(number){
  var string;
  var maximumDigits, i, d, digitPosition, trailingZeros;
  var hasPrintedPoint, isPositive, done;
  var characterReference;
  var c;
  var str;

  string = CreateDynamicArrayCharacters();
  isPositive = true;

  if(number < 0){
    isPositive = false;
    number =  -number;
  }

  if(number == 0){
    DynamicArrayAddCharacter(string, '0');
  }else{
    characterReference = {};

    maximumDigits = nGetMaximumDigitsForDecimal();

    digitPosition = nGetFirstDecimalDigitPosition(number);

    hasPrintedPoint = false;

    if( !isPositive ){
      DynamicArrayAddCharacter(string, '-');
    }

    /* Print leading zeros. */
    if(digitPosition < 0){
      DynamicArrayAddCharacter(string, '0');
      DynamicArrayAddCharacter(string, '.');
      hasPrintedPoint = true;
      for(i = 0; i <  -digitPosition - 1; i = i + 1){
        DynamicArrayAddCharacter(string, '0');
      }
    }

    /* Count trailing zeros */
    trailingZeros = 0;
    done = false;
    for(i = 0; i < maximumDigits &&  !done ; i = i + 1){
      d = nGetDecimalDigitWithFirstDigitPosition(number, digitPosition, maximumDigits - i - 1);
      if(d == 0){
        trailingZeros = trailingZeros + 1;
      }else{
        done = true;
      }
    }

    /* Print number. */
    for(i = 0; i < maximumDigits; i = i + 1){
      d = nGetDecimalDigitWithFirstDigitPosition(number, digitPosition, i);

      if( !hasPrintedPoint  && digitPosition - i + 1 == 0){
        if(maximumDigits - i > trailingZeros){
          DynamicArrayAddCharacter(string, '.');
        }
        hasPrintedPoint = true;
      }

      if(maximumDigits - i <= trailingZeros && hasPrintedPoint){
      }else{
        nGetDecimalDigitCharacterFromNumberWithCheck(d, characterReference);
        c = characterReference.characterValue;
        DynamicArrayAddCharacter(string, c);
      }
    }

    /* Print trailing zeros. */
    for(i = 0; i < digitPosition - maximumDigits + 1; i = i + 1){
      DynamicArrayAddCharacter(string, '0');
    }
  }

  /* Done */
  str = DynamicArrayCharactersToArray(string);
  FreeDynamicArrayCharacters(string);
  return str;
}
function nCreateStringFromNumberWithCheck(number, base, stringRef){
  var string;
  var maximumDigits, i, d, digitPosition, trailingZeros;
  var success, hasPrintedPoint, isPositive, done;
  var characterReference;
  var c;

  string = CreateDynamicArrayCharacters();
  isPositive = true;

  if(number < 0){
    isPositive = false;
    number =  -number;
  }

  if(number == 0){
    DynamicArrayAddCharacter(string, '0');
    success = true;
  }else{
    characterReference = {};

    if(IsInteger(base)){
      success = true;

      maximumDigits = nGetMaximumDigitsForBase(base);

      digitPosition = nGetFirstDigitPosition(number, base);

      hasPrintedPoint = false;

      if( !isPositive ){
        DynamicArrayAddCharacter(string, '-');
      }

      /* Print leading zeros. */
      if(digitPosition < 0){
        DynamicArrayAddCharacter(string, '0');
        DynamicArrayAddCharacter(string, '.');
        hasPrintedPoint = true;
        for(i = 0; i <  -digitPosition - 1; i = i + 1){
          DynamicArrayAddCharacter(string, '0');
        }
      }

      /* Count trailing zeros */
      trailingZeros = 0;
      done = false;
      for(i = 0; i < maximumDigits &&  !done ; i = i + 1){
        d = nGetDigit(number, base, maximumDigits - i - 1);
        if(d == 0){
          trailingZeros = trailingZeros + 1;
        }else{
          done = true;
        }
      }

      /* Print number. */
      for(i = 0; i < maximumDigits && success; i = i + 1){
        d = nGetDigit(number, base, i);

        if(d >= base){
          d = base - 1;
        }

        if( !hasPrintedPoint  && digitPosition - i + 1 == 0){
          if(maximumDigits - i > trailingZeros){
            DynamicArrayAddCharacter(string, '.');
          }
          hasPrintedPoint = true;
        }

        if(maximumDigits - i <= trailingZeros && hasPrintedPoint){
        }else{
          success = nGetSingleDigitCharacterFromNumberWithCheck(d, base, characterReference);
          if(success){
            c = characterReference.characterValue;
            DynamicArrayAddCharacter(string, c);
          }
        }
      }

      if(success){
        /* Print trailing zeros. */
        for(i = 0; i < digitPosition - maximumDigits + 1; i = i + 1){
          DynamicArrayAddCharacter(string, '0');
        }
      }
    }else{
      success = false;
    }
  }

  if(success){
    stringRef.string = DynamicArrayCharactersToArray(string);
    FreeDynamicArrayCharacters(string);
  }

  /* Done */
  return success;
}
function nGetMaximumDigitsForBase(base){
  var t;

  t = Math.pow(10, 15);
  return Math.floor(Math.log10(t)/Math.log10(base));
}
function nGetMaximumDigitsForDecimal(){
  return 15;
}
function nGetFirstDecimalDigitPosition(n){
  var power, m, i;
  var multiply, done;

  n = Math.abs(n);

  if(n != 0){
    if(Math.floor(n) < Math.pow(10, 15)){
      multiply = true;
    }else{
      multiply = false;
    }

    done = false;
    m = 0;
    for(i = 0;  !done ; i = i + 1){
      if(multiply){
        m = n*Math.pow(10, i);
        if(Math.floor(m) >= Math.pow(10, 14)){
          done = true;
        }
      }else{
        m = n/Math.pow(10, i);
        if(Math.floor(m) < Math.pow(10, 15)){
          done = true;
        }
      }
    }

    if(multiply){
      power = 15 - i;
    }else{
      power = 15 + i - 2;
    }

    if(Round(m) >= Math.pow(10, 15)){
      power = power + 1;
    }
  }else{
    power = 1;
  }

  return power;
}
function nGetFirstDigitPosition(n, base){
  var power, m, i, maximumDigits;
  var multiply, done;

  maximumDigits = nGetMaximumDigitsForBase(base);
  n = Math.abs(n);

  if(n != 0){
    if(Math.floor(n) < Math.pow(base, maximumDigits)){
      multiply = true;
    }else{
      multiply = false;
    }

    done = false;
    m = 0;
    for(i = 0;  !done ; i = i + 1){
      if(multiply){
        m = n*Math.pow(base, i);
        if(Math.floor(m) >= Math.pow(base, maximumDigits - 1)){
          done = true;
        }
      }else{
        m = n/Math.pow(base, i);
        if(Math.floor(m) < Math.pow(base, maximumDigits)){
          done = true;
        }
      }
    }

    if(multiply){
      power = maximumDigits - i;
    }else{
      power = maximumDigits + i - 2;
    }

    if(Round(m) >= Math.pow(base, maximumDigits)){
      power = power + 1;
    }
  }else{
    power = 1;
  }

  return power;
}
function nGetSingleDigitCharacterFromNumberWithCheck(c, base, characterReference){
  var numberTable;
  var success;

  numberTable = nGetDigitCharacterTable();

  if(c < base || c < numberTable.length){
    success = true;
    characterReference.characterValue = numberTable[c];
  }else{
    success = false;
  }

  return success;
}
function nGetDecimalDigitCharacterFromNumberWithCheck(c, characterRef){
  var numberTable;
  var success;

  numberTable = "0123456789".split('');

  if(c >= 0 && c < 10){
    success = true;
    characterRef.characterValue = numberTable[c];
  }else{
    success = false;
  }

  return success;
}
function nGetDigitCharacterTable(){
  var numberTable;

  numberTable = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".split('');

  return numberTable;
}
function nGetDecimalDigit(n, index){
  var digitPosition;

  digitPosition = nGetFirstDecimalDigitPosition(n);

  return nGetDecimalDigitWithFirstDigitPosition(n, digitPosition, index);
}
function nGetDecimalDigitWithFirstDigitPosition(n, digitPosition, index){
  var d, e, m, i;

  n = Math.abs(n);

  e = 15 - digitPosition - 1;
  if(e < 0){
    n = Math.round(n/Math.pow(10, Math.abs(e)));
  }else{
    n = Math.round(n*Math.pow(10, e));
  }

  m = n;
  d = 0;
  for(i = 0; i < 15 - index; i = i + 1){
    d = Math.round(m%10);
    m = m - d;
    m = Math.round(m/10);
  }

  return d;
}
function nGetDigit(n, base, index){
  var d, digitPosition, e, m, maximumDigits, i;

  n = Math.abs(n);
  maximumDigits = nGetMaximumDigitsForBase(base);
  digitPosition = nGetFirstDigitPosition(n, base);

  e = maximumDigits - digitPosition - 1;
  if(e < 0){
    n = Math.round(n/Math.pow(base, Math.abs(e)));
  }else{
    n = Math.round(n*Math.pow(base, e));
  }

  m = n;
  d = 0;
  for(i = 0; i < maximumDigits - index; i = i + 1){
    d = Math.round(m%base);
    m = m - d;
    m = Math.round(m/base);
  }

  return d;
}
function nCreateNumberFromDecimalStringWithCheck(string, decimalReference, message){
  return nCreateNumberFromStringWithCheck(string, 10, decimalReference, message);
}
function nCreateNumberFromDecimalString(string){
  var doubleReference;
  var stringReference;
  var number;

  doubleReference = CreateNumberReference(0);
  stringReference = CreateStringReference("".split(''));
  nCreateNumberFromStringWithCheck(string, 10, doubleReference, stringReference);
  number = doubleReference.numberValue;

  delete(doubleReference);
  delete(stringReference);

  return number;
}
function nCreateNumberFromStringWithCheck(string, base, numberReference, message){
  var success;
  var numberIsPositive, exponentIsPositive;
  var beforePoint, afterPoint, exponent;

  numberIsPositive = CreateBooleanReference(true);
  exponentIsPositive = CreateBooleanReference(true);
  beforePoint = {};
  afterPoint = {};
  exponent = {};

  if(base >= 2 && base <= 36){
    success = nExtractPartsFromNumberString(string, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, message);

    if(success){
      numberReference.numberValue = nCreateNumberFromParts(base, numberIsPositive.booleanValue, beforePoint.numberArray, afterPoint.numberArray, exponentIsPositive.booleanValue, exponent.numberArray);
    }
  }else{
    success = false;
    message.string = "Base must be from 2 to 36.".split('');
  }

  return success;
}
function nCreateNumberFromParts(base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent){
  var n, i, p, e;

  n = 0;

  for(i = 0; i < beforePoint.length; i = i + 1){
    p = beforePoint[beforePoint.length - i - 1];

    n = n + p*Math.pow(base, i);
  }

  for(i = 0; i < afterPoint.length; i = i + 1){
    p = afterPoint[i];

    n = n + p/Math.pow(base, i + 1);
  }

  if(exponent.length > 0){
    e = 0;
    for(i = 0; i < exponent.length; i = i + 1){
      p = exponent[exponent.length - i - 1];

      e = e + p*Math.pow(base, i);
    }

    if( !exponentIsPositive ){
      e =  -e;
    }

    n = n*Math.pow(base, e);
  }

  if( !numberIsPositive ){
    n =  -n;
  }

  return n;
}
function nExtractPartsFromNumberString(n, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages){
  var i, j, count;
  var success, done, complete;

  i = 0;
  complete = false;

  if(i < n.length){
    if(n[i] == '-'){
      numberIsPositive.booleanValue = false;
      i = i + 1;
    }else if(n[i] == '+'){
      numberIsPositive.booleanValue = true;
      i = i + 1;
    }

    success = true;
  }else{
    success = false;
    errorMessages.string = "Number cannot have length zero.".split('');
  }

  if(success){
    done = false;
    count = 0;
    for(; i + count < n.length &&  !done ; ){
      if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
        count = count + 1;
      }else{
        done = true;
      }
    }

    if(count >= 1){
      beforePoint.numberArray = [];
      beforePoint.numberArray.length = count;

      for(j = 0; j < count; j = j + 1){
        beforePoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
      }

      i = i + count;

      if(i < n.length){
        success = true;
      }else{
        afterPoint.numberArray = [];
        afterPoint.numberArray.length = 0;
        exponent.numberArray = [];
        exponent.numberArray.length = 0;
        success = true;
        complete = true;
      }
    }else{
      success = false;
      errorMessages.string = "Number must have at least one number after the optional sign.".split('');
    }
  }

  if(success &&  !complete ){
    if(n[i] == '.'){
      i = i + 1;

      if(i < n.length){
        done = false;
        count = 0;
        for(; i + count < n.length &&  !done ; ){
          if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
            count = count + 1;
          }else{
            done = true;
          }
        }

        if(count >= 1){
          afterPoint.numberArray = [];
          afterPoint.numberArray.length = count;

          for(j = 0; j < count; j = j + 1){
            afterPoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
          }

          i = i + count;

          if(i < n.length){
            success = true;
          }else{
            exponent.numberArray = [];
            exponent.numberArray.length = 0;
            success = true;
            complete = true;
          }
        }else{
          success = false;
          errorMessages.string = "There must be at least one digit after the decimal point.".split('');
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the decimal point.".split('');
      }
    }else if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
      if(i < n.length){
        success = true;
        afterPoint.numberArray = [];
        afterPoint.numberArray.length = 0;
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the exponent.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "Expected decimal point or exponent symbol.".split('');
    }
  }

  if(success &&  !complete ){
    if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
      i = i + 1;

      if(i < n.length){
        if(n[i] == '-'){
          exponentIsPositive.booleanValue = false;
          i = i + 1;
        }else if(n[i] == '+'){
          exponentIsPositive.booleanValue = true;
          i = i + 1;
        }

        if(i < n.length){
          done = false;
          count = 0;
          for(; i + count < n.length &&  !done ; ){
            if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
              count = count + 1;
            }else{
              done = true;
            }
          }

          if(count >= 1){
            exponent.numberArray = [];
            exponent.numberArray.length = count;

            for(j = 0; j < count; j = j + 1){
              exponent.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
            }

            i = i + count;

            if(i == n.length){
              success = true;
            }else{
              success = false;
              errorMessages.string = "There cannot be any characters past the exponent of the number.".split('');
            }
          }else{
            success = false;
            errorMessages.string = "There must be at least one digit after the decimal point.".split('');
          }
        }else{
          success = false;
          errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "Expected exponent symbol.".split('');
    }
  }

  return success;
}
function nGetNumberFromNumberCharacterForBase(c, base){
  var numberTable;
  var i;
  var position;

  numberTable = nGetDigitCharacterTable();
  position = 0;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      position = i;
    }
  }

  return position;
}
function nCharacterIsNumberCharacterInBase(c, base){
  var numberTable;
  var i;
  var found;

  numberTable = nGetDigitCharacterTable();
  found = false;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      found = true;
    }
  }

  return found;
}
function nStringToNumberArray(str){
  var numberArrayReference;
  var stringReference;
  var numbers;

  numberArrayReference = {};
  stringReference = {};

  nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

  numbers = numberArrayReference.numberArray;

  delete(numberArrayReference);
  delete(stringReference);

  return numbers;
}
function nStringToNumberArrayWithCheck(str, numberArrayReference, errorMessage){
  var numberStrings;
  var numbers;
  var i;
  var numberString, trimmedNumberString;
  var success;
  var numberReference;

  numberStrings = strSplitByString(str, ",".split(''));

  numbers = [];
  numbers.length = numberStrings.length;
  success = true;
  numberReference = {};

  for(i = 0; i < numberStrings.length; i = i + 1){
    numberString = numberStrings[i].string;
    trimmedNumberString = strTrim(numberString);
    success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
    numbers[i] = numberReference.numberValue;

    FreeStringReference(numberStrings[i]);
    delete(trimmedNumberString);
  }

  delete(numberStrings);
  delete(numberReference);

  numberArrayReference.numberArray = numbers;

  return success;
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;

  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;

  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;

  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;

  /* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors. */
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  var i, j, c;

  c = 1;
  j = 1;
  i = n - k + 1;

  for(; i <= n; ){
    c = c*i;
    c = c/j;

    i = i + 1;
    j = j + 1;
  }

  return c;
}
function Permutations(n, k){
  var i, c;

  c = 1;

  for(i = n - k + 1; i <= n; i = i + 1){
    c = c*i;
  }

  return c;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;

  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function GCDWithSubtraction(a, b){
  var g;

  if(a == 0){
    g = b;
  }else{
    for(; b != 0; ){
      if(a > b){
        a = a - b;
      }else{
        b = b - a;
      }
    }

    g = a;
  }

  return g;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;

  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;

  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;

  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function Gamma(x){
  return LanczosApproximation(x);
}
function LogGamma(x){
  return Math.log(Gamma(x));
}
function LanczosApproximation(z){
  var p;
  var i, y, t, x;

  p = [];
  p.length = 8;
  p[0] = 676.5203681218851;
  p[1] =  -1259.1392167224028;
  p[2] = 771.32342877765313;
  p[3] =  -176.61502916214059;
  p[4] = 12.507343278686905;
  p[5] =  -0.13857109526572012;
  p[6] = 9.9843695780195716e-6;
  p[7] = 1.5056327351493116e-7;

  if(z < 0.5){
    y = Math.PI/(Math.sin(Math.PI*z)*LanczosApproximation(1 - z));
  }else{
    z = z - 1;
    x = 0.99999999999980993;
    for(i = 0; i < p.length; i = i + 1){
      x = x + p[i]/(z + i + 1);
    }
    t = z + p.length - 0.5;
    y = Math.sqrt(2*Math.PI)*Math.pow(t, z + 0.5)*Math.exp( -t)*x;
  }

  return y;
}
function Beta(x, y){
  return Gamma(x)*Gamma(y)/Gamma(x + y);
}
function Sinh(x){
  return (Math.exp(x) - Math.exp( -x))/2;
}
function Cosh(x){
  return (Math.exp(x) + Math.exp( -x))/2;
}
function Tanh(x){
  return Sinh(x)/Cosh(x);
}
function Cot(x){
  return 1/Math.tan(x);
}
function Sec(x){
  return 1/Math.cos(x);
}
function Csc(x){
  return 1/Math.sin(x);
}
function Coth(x){
  return Cosh(x)/Sinh(x);
}
function Sech(x){
  return 1/Cosh(x);
}
function Csch(x){
  return 1/Sinh(x);
}
function Error(x){
  var y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

  if(x == 0){
    y = 0;
  }else if(x < 0){
    y =  -Error( -x);
  }else{
    c1 =  -1.26551223;
    c2 =  +1.00002368;
    c3 =  +0.37409196;
    c4 =  +0.09678418;
    c5 =  -0.18628806;
    c6 =  +0.27886807;
    c7 =  -1.13520398;
    c8 =  +1.48851587;
    c9 =  -0.82215223;
    c10 =  +0.17087277;

    t = 1/(1 + 0.5*Math.abs(x));

    tau = t*Math.exp( -(Math.pow(x, 2)) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

    y = 1 - tau;
  }

  return y;
}
function ErrorInverse(x){
  var y, a, t;

  a = (8*(Math.PI - 3))/(3*Math.PI*(4 - Math.PI));

  t = 2/(Math.PI*a) + Math.log(1 - Math.pow(x, 2))/2;
  y = Sign(x)*Math.sqrt(Math.sqrt(Math.pow(t, 2) - Math.log(1 - Math.pow(x, 2))/a) - t);

  return y;
}
function FallingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x - k);
  }

  return y;
}
function RisingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x + k);
  }

  return y;
}
function Hypergeometric(a, b, c, z, maxIterations, precision){
  var y;

  if(Math.abs(z) >= 0.5){
    y = Math.pow(1 - z,  -a)*HypergeometricDirect(a, c - b, c, z/(z - 1), maxIterations, precision);
  }else{
    y = HypergeometricDirect(a, b, c, z, maxIterations, precision);
  }

  return y;
}
function HypergeometricDirect(a, b, c, z, maxIterations, precision){
  var y, yp, n;
  var done;

  y = 0;
  done = false;

  for(n = 0; n < maxIterations &&  !done ; n = n + 1){
    yp = RisingFactorial(a, n)*RisingFactorial(b, n)/RisingFactorial(c, n)*Math.pow(z, n)/Factorial(n);
    if(Math.abs(yp) < precision){
      done = true;
    }
    y = y + yp;
  }

  return y;
}
function BernouilliNumber(n){
  return AkiyamaTanigawaAlgorithm(n);
}
function AkiyamaTanigawaAlgorithm(n){
  var m, j, B;
  var A;

  A = [];
  A.length = n + 1;

  for(m = 0; m <= n; m = m + 1){
    A[m] = 1/(m + 1);
    for(j = m; j >= 1; j = j - 1){
      A[j - 1] = j*(A[j - 1] - A[j]);
    }
  }

  B = A[0];

  delete(A);

  return B;
}
function StringToNumberArray(string){
  var i;
  var array;

  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function NumberArrayToString(array){
  var i;
  var string;

  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function NumberArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function BooleanArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function StringsEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function FillNumberArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillString(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillBooleanArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillNumberArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillBooleanArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillStringRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyNumberArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyBooleanArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyString(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function IsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function CreateNumberArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillNumberArray(array, value);

  return array;
}
function CreateBooleanArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillBooleanArray(array, value);

  return array;
}
function CreateString(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillString(array, value);

  return array;
}
function SwapElementsOfNumberArray(A, ai, bi){
  var tmp;

  tmp = A[ai];
  A[ai] = A[bi];
  A[bi] = tmp;
}
function SwapElementsOfStringArray(A, ai, bi){
  var tmp;

  tmp = A.stringArray[ai];
  A.stringArray[ai] = A.stringArray[bi];
  A.stringArray[bi] = tmp;
}
function ReverseNumberArray(array){
  var i;

  for(i = 0; i < array.length/2; i = i + 1){
    SwapElementsOfNumberArray(array, i, array.length - i - 1);
  }
}
function charToLowerCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'A'){
    toReturn = 'a';
  }else if(character == 'B'){
    toReturn = 'b';
  }else if(character == 'C'){
    toReturn = 'c';
  }else if(character == 'D'){
    toReturn = 'd';
  }else if(character == 'E'){
    toReturn = 'e';
  }else if(character == 'F'){
    toReturn = 'f';
  }else if(character == 'G'){
    toReturn = 'g';
  }else if(character == 'H'){
    toReturn = 'h';
  }else if(character == 'I'){
    toReturn = 'i';
  }else if(character == 'J'){
    toReturn = 'j';
  }else if(character == 'K'){
    toReturn = 'k';
  }else if(character == 'L'){
    toReturn = 'l';
  }else if(character == 'M'){
    toReturn = 'm';
  }else if(character == 'N'){
    toReturn = 'n';
  }else if(character == 'O'){
    toReturn = 'o';
  }else if(character == 'P'){
    toReturn = 'p';
  }else if(character == 'Q'){
    toReturn = 'q';
  }else if(character == 'R'){
    toReturn = 'r';
  }else if(character == 'S'){
    toReturn = 's';
  }else if(character == 'T'){
    toReturn = 't';
  }else if(character == 'U'){
    toReturn = 'u';
  }else if(character == 'V'){
    toReturn = 'v';
  }else if(character == 'W'){
    toReturn = 'w';
  }else if(character == 'X'){
    toReturn = 'x';
  }else if(character == 'Y'){
    toReturn = 'y';
  }else if(character == 'Z'){
    toReturn = 'z';
  }

  return toReturn;
}
function charToUpperCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'a'){
    toReturn = 'A';
  }else if(character == 'b'){
    toReturn = 'B';
  }else if(character == 'c'){
    toReturn = 'C';
  }else if(character == 'd'){
    toReturn = 'D';
  }else if(character == 'e'){
    toReturn = 'E';
  }else if(character == 'f'){
    toReturn = 'F';
  }else if(character == 'g'){
    toReturn = 'G';
  }else if(character == 'h'){
    toReturn = 'H';
  }else if(character == 'i'){
    toReturn = 'I';
  }else if(character == 'j'){
    toReturn = 'J';
  }else if(character == 'k'){
    toReturn = 'K';
  }else if(character == 'l'){
    toReturn = 'L';
  }else if(character == 'm'){
    toReturn = 'M';
  }else if(character == 'n'){
    toReturn = 'N';
  }else if(character == 'o'){
    toReturn = 'O';
  }else if(character == 'p'){
    toReturn = 'P';
  }else if(character == 'q'){
    toReturn = 'Q';
  }else if(character == 'r'){
    toReturn = 'R';
  }else if(character == 's'){
    toReturn = 'S';
  }else if(character == 't'){
    toReturn = 'T';
  }else if(character == 'u'){
    toReturn = 'U';
  }else if(character == 'v'){
    toReturn = 'V';
  }else if(character == 'w'){
    toReturn = 'W';
  }else if(character == 'x'){
    toReturn = 'X';
  }else if(character == 'y'){
    toReturn = 'Y';
  }else if(character == 'z'){
    toReturn = 'Z';
  }

  return toReturn;
}
function charIsUpperCase(character){
  var isUpper;

  isUpper = true;
  if(character == 'A'){
  }else if(character == 'B'){
  }else if(character == 'C'){
  }else if(character == 'D'){
  }else if(character == 'E'){
  }else if(character == 'F'){
  }else if(character == 'G'){
  }else if(character == 'H'){
  }else if(character == 'I'){
  }else if(character == 'J'){
  }else if(character == 'K'){
  }else if(character == 'L'){
  }else if(character == 'M'){
  }else if(character == 'N'){
  }else if(character == 'O'){
  }else if(character == 'P'){
  }else if(character == 'Q'){
  }else if(character == 'R'){
  }else if(character == 'S'){
  }else if(character == 'T'){
  }else if(character == 'U'){
  }else if(character == 'V'){
  }else if(character == 'W'){
  }else if(character == 'X'){
  }else if(character == 'Y'){
  }else if(character == 'Z'){
  }else{
    isUpper = false;
  }

  return isUpper;
}
function charIsLowerCase(character){
  var isLower;

  isLower = true;
  if(character == 'a'){
  }else if(character == 'b'){
  }else if(character == 'c'){
  }else if(character == 'd'){
  }else if(character == 'e'){
  }else if(character == 'f'){
  }else if(character == 'g'){
  }else if(character == 'h'){
  }else if(character == 'i'){
  }else if(character == 'j'){
  }else if(character == 'k'){
  }else if(character == 'l'){
  }else if(character == 'm'){
  }else if(character == 'n'){
  }else if(character == 'o'){
  }else if(character == 'p'){
  }else if(character == 'q'){
  }else if(character == 'r'){
  }else if(character == 's'){
  }else if(character == 't'){
  }else if(character == 'u'){
  }else if(character == 'v'){
  }else if(character == 'w'){
  }else if(character == 'x'){
  }else if(character == 'y'){
  }else if(character == 'z'){
  }else{
    isLower = false;
  }

  return isLower;
}
function charIsLetter(character){
  return charIsUpperCase(character) || charIsLowerCase(character);
}
function charIsNumber(character){
  var isNumberx;

  isNumberx = true;
  if(character == '0'){
  }else if(character == '1'){
  }else if(character == '2'){
  }else if(character == '3'){
  }else if(character == '4'){
  }else if(character == '5'){
  }else if(character == '6'){
  }else if(character == '7'){
  }else if(character == '8'){
  }else if(character == '9'){
  }else{
    isNumberx = false;
  }

  return isNumberx;
}
function charIsWhiteSpace(character){
  var isWhiteSpacex;

  isWhiteSpacex = true;
  if(character == ' '){
  }else if(character == '\t'){
  }else if(character == '\n'){
  }else if(character == '\r'){
  }else{
    isWhiteSpacex = false;
  }

  return isWhiteSpacex;
}
function charIsSymbol(character){
  var isSymbolx;

  isSymbolx = true;
  if(character == '!'){
  }else if(character == '\"'){
  }else if(character == '#'){
  }else if(character == '$'){
  }else if(character == '%'){
  }else if(character == '&'){
  }else if(character == '\''){
  }else if(character == '('){
  }else if(character == ')'){
  }else if(character == '*'){
  }else if(character == '+'){
  }else if(character == ','){
  }else if(character == '-'){
  }else if(character == '.'){
  }else if(character == '/'){
  }else if(character == ':'){
  }else if(character == ';'){
  }else if(character == '<'){
  }else if(character == '='){
  }else if(character == '>'){
  }else if(character == '?'){
  }else if(character == '@'){
  }else if(character == '['){
  }else if(character == '\\'){
  }else if(character == ']'){
  }else if(character == '^'){
  }else if(character == '_'){
  }else if(character == '`'){
  }else if(character == '{'){
  }else if(character == '|'){
  }else if(character == '}'){
  }else if(character == '~'){
  }else{
    isSymbolx = false;
  }

  return isSymbolx;
}
function charCharacterIsBefore(a, b){
  var ad, bd;

  ad = a.charCodeAt(0);
  bd = b.charCodeAt(0);

  return ad < bd;
}
function charDecimalDigitToCharacter(digit){
  var c;
  if(digit == 1){
    c = '1';
  }else if(digit == 2){
    c = '2';
  }else if(digit == 3){
    c = '3';
  }else if(digit == 4){
    c = '4';
  }else if(digit == 5){
    c = '5';
  }else if(digit == 6){
    c = '6';
  }else if(digit == 7){
    c = '7';
  }else if(digit == 8){
    c = '8';
  }else if(digit == 9){
    c = '9';
  }else{
    c = '0';
  }
  return c;
}
function charCharacterToDecimalDigit(c){
  var digit;

  if(c == '1'){
    digit = 1;
  }else if(c == '2'){
    digit = 2;
  }else if(c == '3'){
    digit = 3;
  }else if(c == '4'){
    digit = 4;
  }else if(c == '5'){
    digit = 5;
  }else if(c == '6'){
    digit = 6;
  }else if(c == '7'){
    digit = 7;
  }else if(c == '8'){
    digit = 8;
  }else if(c == '9'){
    digit = 9;
  }else{
    digit = 0;
  }

  return digit;
}
function AddNumber(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddNumberRef(list, i){
  list.numberArray = AddNumber(list.numberArray, i);
}
function RemoveNumber(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetNumberRef(list, i){
  return list.numberArray[i];
}
function RemoveNumberRef(list, i){
  list.numberArray = RemoveNumber(list.numberArray, i);
}
function AddString(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;

  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddStringRef(list, i){
  list.stringArray = AddString(list.stringArray, i);
}
function RemoveString(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetStringRef(list, i){
  return list.stringArray[i];
}
function RemoveStringRef(list, i){
  list.stringArray = RemoveString(list.stringArray, i);
}
function CreateDynamicArrayCharacters(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayCharactersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddCharacter(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayCharactersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayCharactersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayCharactersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayCharactersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayCharactersIndex(da, index){
  return da.array[index];
}
function DynamicArrayCharactersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertCharacter(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayCharactersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArrayCharacterSet(da, index, value){
  var success;

  if(index < da.lengthx){
    da.array[index] = value;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function DynamicArrayRemoveCharacter(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayCharactersDecreaseSizeNecessary(da)){
    DynamicArrayCharactersDecreaseSize(da);
  }
}
function FreeDynamicArrayCharacters(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayCharactersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayCharactersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayCharactersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayCharacters(array){
  var da;

  da = {};
  da.array = CopyString(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayCharactersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayCharactersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListCharacter();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddCharacter(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayCharacters(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListCharactersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function AddBoolean(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddBooleanRef(list, i){
  list.booleanArray = AddBoolean(list.booleanArray, i);
}
function RemoveBoolean(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetBooleanRef(list, i){
  return list.booleanArray[i];
}
function RemoveDecimalRef(list, i){
  list.booleanArray = RemoveBoolean(list.booleanArray, i);
}
function CreateLinkedListString(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddString(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListStringsToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListStringsLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = {};
    array[i].string = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListStringsLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListString(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateLinkedListNumbers(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function CreateLinkedListNumbersArray(lengthx){
  var lls;
  var i;

  lls = [];
  lls.length = lengthx;
  for(i = 0; i < lls.length; i = i + 1){
    lls[i] = CreateLinkedListNumbers();
  }

  return lls;
}
function LinkedListAddNumber(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListNumbersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function LinkedListNumbersIndex(ll, index){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  return node.value;
}
function LinkedListInsertNumber(ll, index, value){
  var i;
  var node, tmp;

  if(index == 0){
    tmp = ll.first;
    ll.first = {};
    ll.first.next = tmp;
    ll.first.value = value;
    ll.first.end = false;
  }else{
    node = ll.first;
    for(i = 0; i < index - 1; i = i + 1){
      node = node.next;
    }

    tmp = node.next;
    node.next = {};
    node.next.next = tmp;
    node.next.value = value;
    node.next.end = false;
  }
}
function LinkedListSet(ll, index, value){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  node.next.value = value;
}
function LinkedListRemoveNumber(ll, index){
  var i;
  var node, prev;

  node = ll.first;
  prev = ll.first;

  for(i = 0; i < index; i = i + 1){
    prev = node;
    node = node.next;
  }

  if(index == 0){
    ll.first = prev.next;
  }
  if( !prev.next.end ){
    prev.next = prev.next.next;
  }
}
function FreeLinkedListNumbers(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function FreeLinkedListNumbersArray(lls){
  var i;

  for(i = 0; i < lls.length; i = i + 1){
    FreeLinkedListNumbers(lls[i]);
  }
  delete(lls);
}
function LinkedListNumbersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListNumbersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function ArrayToLinkedListNumbers(array){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < array.length; i = i + 1){
    LinkedListAddNumber(ll, array[i]);
  }

  return ll;
}
function LinkedListNumbersEqual(a, b){
  var equal, done;
  var an, bn;

  an = a.first;
  bn = b.first;

  equal = true;
  done = false;
  for(; equal &&  !done ; ){
    if(an.end == bn.end){
      if(an.end){
        done = true;
      }else if(an.value == bn.value){
        an = an.next;
        bn = bn.next;
      }else{
        equal = false;
      }
    }else{
      equal = false;
    }
  }

  return equal;
}
function CreateLinkedListCharacter(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddCharacter(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListCharactersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListCharactersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListCharactersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListCharacter(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function LinkedListCharactersAddString(ll, str){
  var i;

  for(i = 0; i < str.length; i = i + 1){
    LinkedListAddCharacter(ll, str[i]);
  }
}
function CreateDynamicArrayNumbers(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayNumbersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddNumber(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumbersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayNumbersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersIndex(da, index){
  return da.array[index];
}
function DynamicArrayNumbersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertNumber(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumberSet(da, index, value){
  var success;

  if(index < da.lengthx){
    da.array[index] = value;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function DynamicArrayRemoveNumber(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayNumbersDecreaseSizeNecessary(da)){
    DynamicArrayNumbersDecreaseSize(da);
  }
}
function FreeDynamicArrayNumbers(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayNumbersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayNumbersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  /*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
         */
  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayNumbersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayNumbers(array){
  var da;

  da = {};
  da.array = CopyNumberArray(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayNumbersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayNumbersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddNumber(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayNumbers(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListNumbersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function DynamicArrayNumbersIndexOf(arr, n, foundReference){
  var found;
  var i;

  found = false;
  for(i = 0; i < arr.lengthx &&  !found ; i = i + 1){
    if(arr.array[i] == n){
      found = true;
    }
  }
  if( !found ){
    i =  -1;
  }else{
    i = i - 1;
  }

  foundReference.booleanValue = found;

  return i;
}
function DynamicArrayNumbersIsInArray(arr, n){
  var found;
  var i;

  found = false;
  for(i = 0; i < arr.lengthx &&  !found ; i = i + 1){
    if(arr.array[i] == n){
      found = true;
    }
  }

  return found;
}
function AddCharacter(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddCharacterRef(list, i){
  list.string = AddCharacter(list.string, i);
}
function RemoveCharacter(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }

  return newlist;
}
function GetCharacterRef(list, i){
  return list.string[i];
}
function RemoveCharacterRef(list, i){
  list.string = RemoveCharacter(list.string, i);
}


module.exports = {
	ResponseCacheInitialize: ResponseCacheInitialize,
	ResponseCacheIteration: ResponseCacheIteration,
	ModulusHash: ModulusHash,
	Check: Check,
	Send: Send,
	Receive: Receive,
	Call: Call,
	ServerCheck: ServerCheck,
	ServerSend: ServerSend,
	ServerReceive: ServerReceive,
	ReadXbytes: ReadXbytes,
	Read4bytesBE: Read4bytesBE,
	Read2bytesBE: Read2bytesBE,
	ReadByte: ReadByte,
	Read4bytesLE: Read4bytesLE,
	WriteByte: WriteByte,
	Write2BytesLE: Write2BytesLE,
	Write4BytesLE: Write4BytesLE,
	Write2BytesBE: Write2BytesBE,
	Write4BytesBE: Write4BytesBE,
	WriteStringBytes: WriteStringBytes,
	BytesRound: BytesRound,
	ByteArrayToNumberArray: ByteArrayToNumberArray,
	NumberArrayToByteArray: NumberArrayToByteArray,
	ByteArraysEqual: ByteArraysEqual,
	CopyByteArray: CopyByteArray,
	ByteArrayLength: ByteArrayLength,
	CreateAndFillByteArray: CreateAndFillByteArray,
	CreateByteArray: CreateByteArray,
	SetByte: SetByte,
	GetByte: GetByte,
	AssertByteArraysEqual: AssertByteArraysEqual,
	FreeByteArray: FreeByteArray,
	CopyByteArrayRange: CopyByteArrayRange,
	BytesToTextBase16: BytesToTextBase16,
	TextToBytesBase16WithCheck: TextToBytesBase16WithCheck,
	TextToBytesBase16: TextToBytesBase16,
	FreeBase64ByteCombinations: FreeBase64ByteCombinations,
	GenerateBase16ByteCombinations: GenerateBase16ByteCombinations,
	JSONCompare: JSONCompare,
	JSONCompareElements: JSONCompareElements,
	JSONCompareArrays: JSONCompareArrays,
	JSONCompareObjects: JSONCompareObjects,
	FreeStringReferenceArray: FreeStringReferenceArray,
	WriteJSON: WriteJSON,
	WriteBooleanValue: WriteBooleanValue,
	WriteNumber: WriteNumber,
	WriteArray: WriteArray,
	WriteString: WriteString,
	JSONEscapeString: JSONEscapeString,
	JSONEscapedStringLength: JSONEscapedStringLength,
	JSONEscapeCharacter: JSONEscapeCharacter,
	JSONMustBeEscaped: JSONMustBeEscaped,
	WriteObject: WriteObject,
	WriteJSONFromStruct: WriteJSONFromStruct,
	WriteJSONFromArray: WriteJSONFromArray,
	ReadJSON: ReadJSON,
	GetJSONValue: GetJSONValue,
	GetJSONValueRecursive: GetJSONValueRecursive,
	GetJSONObject: GetJSONObject,
	GetJSONArray: GetJSONArray,
	JSONTokenize: JSONTokenize,
	GetJSONNumberToken: GetJSONNumberToken,
	IsValidJSONNumber: IsValidJSONNumber,
	IsValidJSONNumberAfterSign: IsValidJSONNumberAfterSign,
	IsValidJSONNumberAdvancePastDigits: IsValidJSONNumberAdvancePastDigits,
	IsValidJSONNumberFromDotOrExponent: IsValidJSONNumberFromDotOrExponent,
	IsValidJSONNumberFromExponent: IsValidJSONNumberFromExponent,
	IsJSONNumberCharacter: IsJSONNumberCharacter,
	GetJSONPrimitiveName: GetJSONPrimitiveName,
	GetJSONString: GetJSONString,
	IsValidJSONString: IsValidJSONString,
	IsValidJSONStringInJSON: IsValidJSONStringInJSON,
	IsJSONIllegalControllCharacter: IsJSONIllegalControllCharacter,
	IsValidJSON: IsValidJSON,
	ComputeJSONStringLength: ComputeJSONStringLength,
	ComputeJSONBooleanStringLength: ComputeJSONBooleanStringLength,
	ComputeJSONNumberStringLength: ComputeJSONNumberStringLength,
	ComputeJSONArrayStringLength: ComputeJSONArrayStringLength,
	ComputeJSONObjectStringLength: ComputeJSONObjectStringLength,
	UnicodeToUTF8: UnicodeToUTF8,
	UTF8ToUnicode: UTF8ToUnicode,
	UTF16ToUnicode: UTF16ToUnicode,
	UnicodeToUTF16: UnicodeToUTF16,
	UTF8ToUTF16: UTF8ToUTF16,
	UTF16ToUTF8: UTF16ToUTF8,
	IsValidUnicode: IsValidUnicode,
	IsValidUtf16: IsValidUtf16,
	IsValidUtf8: IsValidUtf8,
	CreateArrayData: CreateArrayData,
	CreateStructData: CreateStructData,
	CreateStructure: CreateStructure,
	CreateNumberData: CreateNumberData,
	CreateBooleanData: CreateBooleanData,
	CreateStringData: CreateStringData,
	CreateNoTypeData: CreateNoTypeData,
	AddStructToArray: AddStructToArray,
	AddArrayToArray: AddArrayToArray,
	AddNumberToArray: AddNumberToArray,
	AddBooleanToArray: AddBooleanToArray,
	AddStringToArray: AddStringToArray,
	AddDataToArray: AddDataToArray,
	StructKeys: StructKeys,
	StructHasKey: StructHasKey,
	StructKeyIndex: StructKeyIndex,
	GetStructKeys: GetStructKeys,
	GetStructFromStruct: GetStructFromStruct,
	GetArrayFromStruct: GetArrayFromStruct,
	GetNumberFromStruct: GetNumberFromStruct,
	GetBooleanFromStruct: GetBooleanFromStruct,
	GetStringFromStruct: GetStringFromStruct,
	GetDataFromStruct: GetDataFromStruct,
	GetDataFromStructWithCheck: GetDataFromStructWithCheck,
	AddStructToStruct: AddStructToStruct,
	AddArrayToStruct: AddArrayToStruct,
	AddNumberToStruct: AddNumberToStruct,
	AddBooleanToStruct: AddBooleanToStruct,
	AddStringToStruct: AddStringToStruct,
	AddDataToStruct: AddDataToStruct,
	FreeData: FreeData,
	FreeArray: FreeArray,
	DataTypeEquals: DataTypeEquals,
	IsStructure: IsStructure,
	IsArray: IsArray,
	IsNumber: IsNumber,
	IsBoolean: IsBoolean,
	IsString: IsString,
	IsNoType: IsNoType,
	CreateArray: CreateArray,
	CreateArrayWithInitialCapacity: CreateArrayWithInitialCapacity,
	ArrayAdd: ArrayAdd,
	ArrayIncreaseSize: ArrayIncreaseSize,
	ArrayDecreaseSizeNecessary: ArrayDecreaseSizeNecessary,
	ArrayDecreaseSize: ArrayDecreaseSize,
	ArrayIndex: ArrayIndex,
	ArrayLength: ArrayLength,
	ArrayInsert: ArrayInsert,
	ArraySet: ArraySet,
	ArrayRemove: ArrayRemove,
	ToStaticArray: ToStaticArray,
	ToStaticNumberArray: ToStaticNumberArray,
	ToStaticBooleanArray: ToStaticBooleanArray,
	ToStaticStringArray: ToStaticStringArray,
	ToStaticArrayArray: ToStaticArrayArray,
	ToStaticStructArray: ToStaticStructArray,
	StaticArrayToArrayWithOptimalSize: StaticArrayToArrayWithOptimalSize,
	StaticArrayToArray: StaticArrayToArray,
	ArraysEqual: ArraysEqual,
	DataArraysEqual: DataArraysEqual,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	strWriteStringToStingStream: strWriteStringToStingStream,
	strWriteCharacterToStingStream: strWriteCharacterToStingStream,
	strWriteBooleanToStingStream: strWriteBooleanToStingStream,
	strSubstringWithCheck: strSubstringWithCheck,
	strSubstring: strSubstring,
	strAppendString: strAppendString,
	strConcatenateString: strConcatenateString,
	strAppendCharacter: strAppendCharacter,
	strConcatenateCharacter: strConcatenateCharacter,
	strSplitByCharacter: strSplitByCharacter,
	strIndexOfCharacter: strIndexOfCharacter,
	strSubstringEqualsWithCheck: strSubstringEqualsWithCheck,
	strSubstringEquals: strSubstringEquals,
	strIndexOfString: strIndexOfString,
	strContainsCharacter: strContainsCharacter,
	strContainsString: strContainsString,
	strToUpperCase: strToUpperCase,
	strToLowerCase: strToLowerCase,
	strEqualsIgnoreCase: strEqualsIgnoreCase,
	strReplaceString: strReplaceString,
	strReplaceCharacterToNew: strReplaceCharacterToNew,
	strReplaceCharacter: strReplaceCharacter,
	strTrim: strTrim,
	strStartsWith: strStartsWith,
	strEndsWith: strEndsWith,
	strSplitByString: strSplitByString,
	strStringIsBefore: strStringIsBefore,
	strJoinStringsWithSeparator: strJoinStringsWithSeparator,
	strJoinStrings: strJoinStrings,
	nCreateStringScientificNotationDecimalFromNumber: nCreateStringScientificNotationDecimalFromNumber,
	nCreateStringDecimalFromNumber: nCreateStringDecimalFromNumber,
	nCreateStringFromNumberWithCheck: nCreateStringFromNumberWithCheck,
	nGetMaximumDigitsForBase: nGetMaximumDigitsForBase,
	nGetMaximumDigitsForDecimal: nGetMaximumDigitsForDecimal,
	nGetFirstDecimalDigitPosition: nGetFirstDecimalDigitPosition,
	nGetFirstDigitPosition: nGetFirstDigitPosition,
	nGetSingleDigitCharacterFromNumberWithCheck: nGetSingleDigitCharacterFromNumberWithCheck,
	nGetDecimalDigitCharacterFromNumberWithCheck: nGetDecimalDigitCharacterFromNumberWithCheck,
	nGetDigitCharacterTable: nGetDigitCharacterTable,
	nGetDecimalDigit: nGetDecimalDigit,
	nGetDecimalDigitWithFirstDigitPosition: nGetDecimalDigitWithFirstDigitPosition,
	nGetDigit: nGetDigit,
	nCreateNumberFromDecimalStringWithCheck: nCreateNumberFromDecimalStringWithCheck,
	nCreateNumberFromDecimalString: nCreateNumberFromDecimalString,
	nCreateNumberFromStringWithCheck: nCreateNumberFromStringWithCheck,
	nCreateNumberFromParts: nCreateNumberFromParts,
	nExtractPartsFromNumberString: nExtractPartsFromNumberString,
	nGetNumberFromNumberCharacterForBase: nGetNumberFromNumberCharacterForBase,
	nCharacterIsNumberCharacterInBase: nCharacterIsNumberCharacterInBase,
	nStringToNumberArray: nStringToNumberArray,
	nStringToNumberArrayWithCheck: nStringToNumberArrayWithCheck,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	Permutations: Permutations,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	GCDWithSubtraction: GCDWithSubtraction,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	Gamma: Gamma,
	LogGamma: LogGamma,
	LanczosApproximation: LanczosApproximation,
	Beta: Beta,
	Sinh: Sinh,
	Cosh: Cosh,
	Tanh: Tanh,
	Cot: Cot,
	Sec: Sec,
	Csc: Csc,
	Coth: Coth,
	Sech: Sech,
	Csch: Csch,
	Error: Error,
	ErrorInverse: ErrorInverse,
	FallingFactorial: FallingFactorial,
	RisingFactorial: RisingFactorial,
	Hypergeometric: Hypergeometric,
	HypergeometricDirect: HypergeometricDirect,
	BernouilliNumber: BernouilliNumber,
	AkiyamaTanigawaAlgorithm: AkiyamaTanigawaAlgorithm,
	StringToNumberArray: StringToNumberArray,
	NumberArrayToString: NumberArrayToString,
	NumberArraysEqual: NumberArraysEqual,
	BooleanArraysEqual: BooleanArraysEqual,
	StringsEqual: StringsEqual,
	FillNumberArray: FillNumberArray,
	FillString: FillString,
	FillBooleanArray: FillBooleanArray,
	FillNumberArrayRange: FillNumberArrayRange,
	FillBooleanArrayRange: FillBooleanArrayRange,
	FillStringRange: FillStringRange,
	CopyNumberArray: CopyNumberArray,
	CopyBooleanArray: CopyBooleanArray,
	CopyString: CopyString,
	CopyNumberArrayRange: CopyNumberArrayRange,
	CopyBooleanArrayRange: CopyBooleanArrayRange,
	CopyStringRange: CopyStringRange,
	IsLastElement: IsLastElement,
	CreateNumberArray: CreateNumberArray,
	CreateBooleanArray: CreateBooleanArray,
	CreateString: CreateString,
	SwapElementsOfNumberArray: SwapElementsOfNumberArray,
	SwapElementsOfStringArray: SwapElementsOfStringArray,
	ReverseNumberArray: ReverseNumberArray,
	charToLowerCase: charToLowerCase,
	charToUpperCase: charToUpperCase,
	charIsUpperCase: charIsUpperCase,
	charIsLowerCase: charIsLowerCase,
	charIsLetter: charIsLetter,
	charIsNumber: charIsNumber,
	charIsWhiteSpace: charIsWhiteSpace,
	charIsSymbol: charIsSymbol,
	charCharacterIsBefore: charCharacterIsBefore,
	charDecimalDigitToCharacter: charDecimalDigitToCharacter,
	charCharacterToDecimalDigit: charCharacterToDecimalDigit,
	AddNumber: AddNumber,
	AddNumberRef: AddNumberRef,
	RemoveNumber: RemoveNumber,
	GetNumberRef: GetNumberRef,
	RemoveNumberRef: RemoveNumberRef,
	AddString: AddString,
	AddStringRef: AddStringRef,
	RemoveString: RemoveString,
	GetStringRef: GetStringRef,
	RemoveStringRef: RemoveStringRef,
	CreateDynamicArrayCharacters: CreateDynamicArrayCharacters,
	CreateDynamicArrayCharactersWithInitialCapacity: CreateDynamicArrayCharactersWithInitialCapacity,
	DynamicArrayAddCharacter: DynamicArrayAddCharacter,
	DynamicArrayCharactersIncreaseSize: DynamicArrayCharactersIncreaseSize,
	DynamicArrayCharactersDecreaseSizeNecessary: DynamicArrayCharactersDecreaseSizeNecessary,
	DynamicArrayCharactersDecreaseSize: DynamicArrayCharactersDecreaseSize,
	DynamicArrayCharactersIndex: DynamicArrayCharactersIndex,
	DynamicArrayCharactersLength: DynamicArrayCharactersLength,
	DynamicArrayInsertCharacter: DynamicArrayInsertCharacter,
	DynamicArrayCharacterSet: DynamicArrayCharacterSet,
	DynamicArrayRemoveCharacter: DynamicArrayRemoveCharacter,
	FreeDynamicArrayCharacters: FreeDynamicArrayCharacters,
	DynamicArrayCharactersToArray: DynamicArrayCharactersToArray,
	ArrayToDynamicArrayCharactersWithOptimalSize: ArrayToDynamicArrayCharactersWithOptimalSize,
	ArrayToDynamicArrayCharacters: ArrayToDynamicArrayCharacters,
	DynamicArrayCharactersEqual: DynamicArrayCharactersEqual,
	DynamicArrayCharactersToLinkedList: DynamicArrayCharactersToLinkedList,
	LinkedListToDynamicArrayCharacters: LinkedListToDynamicArrayCharacters,
	AddBoolean: AddBoolean,
	AddBooleanRef: AddBooleanRef,
	RemoveBoolean: RemoveBoolean,
	GetBooleanRef: GetBooleanRef,
	RemoveDecimalRef: RemoveDecimalRef,
	CreateLinkedListString: CreateLinkedListString,
	LinkedListAddString: LinkedListAddString,
	LinkedListStringsToArray: LinkedListStringsToArray,
	LinkedListStringsLength: LinkedListStringsLength,
	FreeLinkedListString: FreeLinkedListString,
	CreateLinkedListNumbers: CreateLinkedListNumbers,
	CreateLinkedListNumbersArray: CreateLinkedListNumbersArray,
	LinkedListAddNumber: LinkedListAddNumber,
	LinkedListNumbersLength: LinkedListNumbersLength,
	LinkedListNumbersIndex: LinkedListNumbersIndex,
	LinkedListInsertNumber: LinkedListInsertNumber,
	LinkedListSet: LinkedListSet,
	LinkedListRemoveNumber: LinkedListRemoveNumber,
	FreeLinkedListNumbers: FreeLinkedListNumbers,
	FreeLinkedListNumbersArray: FreeLinkedListNumbersArray,
	LinkedListNumbersToArray: LinkedListNumbersToArray,
	ArrayToLinkedListNumbers: ArrayToLinkedListNumbers,
	LinkedListNumbersEqual: LinkedListNumbersEqual,
	CreateLinkedListCharacter: CreateLinkedListCharacter,
	LinkedListAddCharacter: LinkedListAddCharacter,
	LinkedListCharactersToArray: LinkedListCharactersToArray,
	LinkedListCharactersLength: LinkedListCharactersLength,
	FreeLinkedListCharacter: FreeLinkedListCharacter,
	LinkedListCharactersAddString: LinkedListCharactersAddString,
	CreateDynamicArrayNumbers: CreateDynamicArrayNumbers,
	CreateDynamicArrayNumbersWithInitialCapacity: CreateDynamicArrayNumbersWithInitialCapacity,
	DynamicArrayAddNumber: DynamicArrayAddNumber,
	DynamicArrayNumbersIncreaseSize: DynamicArrayNumbersIncreaseSize,
	DynamicArrayNumbersDecreaseSizeNecessary: DynamicArrayNumbersDecreaseSizeNecessary,
	DynamicArrayNumbersDecreaseSize: DynamicArrayNumbersDecreaseSize,
	DynamicArrayNumbersIndex: DynamicArrayNumbersIndex,
	DynamicArrayNumbersLength: DynamicArrayNumbersLength,
	DynamicArrayInsertNumber: DynamicArrayInsertNumber,
	DynamicArrayNumberSet: DynamicArrayNumberSet,
	DynamicArrayRemoveNumber: DynamicArrayRemoveNumber,
	FreeDynamicArrayNumbers: FreeDynamicArrayNumbers,
	DynamicArrayNumbersToArray: DynamicArrayNumbersToArray,
	ArrayToDynamicArrayNumbersWithOptimalSize: ArrayToDynamicArrayNumbersWithOptimalSize,
	ArrayToDynamicArrayNumbers: ArrayToDynamicArrayNumbers,
	DynamicArrayNumbersEqual: DynamicArrayNumbersEqual,
	DynamicArrayNumbersToLinkedList: DynamicArrayNumbersToLinkedList,
	LinkedListToDynamicArrayNumbers: LinkedListToDynamicArrayNumbers,
	DynamicArrayNumbersIndexOf: DynamicArrayNumbersIndexOf,
	DynamicArrayNumbersIsInArray: DynamicArrayNumbersIsInArray,
	AddCharacter: AddCharacter,
	AddCharacterRef: AddCharacterRef,
	RemoveCharacter: RemoveCharacter,
	GetCharacterRef: GetCharacterRef,
	RemoveCharacterRef: RemoveCharacterRef,
	test: test
};